/***********************************************
** auth.cpp            horseshoeFTPd          **
** Nick Byra                                  **
** 21/03/06                                   **
**                                            **
***********************************************/


#include "auth.h"
#include "consts.h"
#include "xml_manager.h"
#include "mem_manage.h"


//check to see if the server is full
int authServerFull(ftp_options *serverOptions)
{
	//if the amount of current users is larger then the max users the server can old
	//we return the error code of ERR_SERVERFULL
	//if the server is not full then we return and AUTHORIZED
	if(serverOptions->getCurrentUsers() >= serverOptions->getMaxusers())
	{
		return ERR_SERVERFULL;
	}

	return AUTHORIZED;
}


//check to see if a user is over is max allowed number of concurrent logins
//if he is we return the error ERR_MAXUSERSESSIONS
//if the user is not then we give the AUTHORIZED
int authUserSessions(ftp_options *serverOptions, user *sUser)
{
	if(serverOptions->getNumConnections(sUser) > sUser->getmaxLogins())
	{
		return ERR_MAXUSERSESSIONS;

	}

	return AUTHORIZED;

}


//check to see if the ip matches the ipmask give
//this accepts things like wild cards *
//and ranges like 015-130
//ranges must be 3 digit numbers seperated by a -
//this function assumes everything is formated correctly
//error check must take place elsewhere
bool checkIP(char *IP, char *IPmask)
{
	
	//setup char* to be used with strtok_s  for ip and ipmask
	char *t1 = NULL;
	char *t2 = NULL;
	char *iptoken = NULL;
	char *masktoken = NULL;

	//tokenize the strings
	iptoken  = strtok_s(IP,".",&t1);
	masktoken = strtok_s(IPmask,".",&t2);
	//looping through all four segments
	for(int i=0;i<4;i++)
	{

		//if the ip mask is just a number
		//say 124 for example
		if(strlen(masktoken) == 3)
		{

			//check to see if its a match with the ip
			if(!strcmp(iptoken,masktoken))
			{
				//pass
			}
			else
			{
				//return false if it is not a match with the ip
				return false;
			}
		}
		else
		{
			//if the ipmask is a range
			if(strlen(masktoken) == 7)
			{
				//set the part of the string that is - to null
				//this makes the string just the first three numbers
				masktoken[3]='\0';
				//check to see if the ip is greater then or equal to the first three numbers
				if(atoi(masktoken)<=atoi(iptoken))
				{
					//move the last 3 numbers to the place of the first three numbers to check them
					masktoken[0] = masktoken[4];
					masktoken[1] = masktoken[5];
					masktoken[2] = masktoken[6];
					//checking to see if the ip is less the or equal to the mask
					if(atoi(masktoken)>=atoi(iptoken))
					{
					}
					else
					{
						//return false if it is out of the range					
						return false;
					}
				}
				else
				{
					//return false if it is out of the range	
					return false;
				}

			}
			else
			{
				//if we have a wildcard *
				if(strlen(masktoken) == 1)
				{
					//this passes aslong as it is *
					//it should never be anything other the a  * though
					if(!strcmp(masktoken,"*"))
					{
						//pass
					}
					else
					{
						//return false if the ipmask is of length one and is smomething other then *
						//this should never happen
						return false;	
					}
				}
			}


		}
		//move to the next tokens and repeat
		iptoken  = strtok_s(NULL,".",&t1);
		masktoken = strtok_s(NULL,".",&t2);

	}
	//this is the end of the loop so if we did not fail
	//this means that the ip mask includes the ip so we return true
	return true;
}


//checks to see if the acount is active
//nohing more the a wapper for the getactive() user has 
bool authAccountActive(user *sUser)
{

	return sUser->getactive();

}


//this functions sees if an ip given as a unsigned long
//is in the ip rules of a user
//if they are we are gonna return true and if not return false
bool authUserIP(u_long IPul, user *sUser)
{
	//to easily get a string from the long we need to put it in a sockaddr
	sockaddr_in b;
	b.sin_addr.S_un.S_addr = IPul;
	char *IP = inet_ntoa(b.sin_addr);
	char nIP[16];
	char mIP[32];
	strcpy_s(nIP,16,IP);

	//get the first node in the ip rules of the user and assign it to a
	IPrules *a = sUser->getiprules();
	//check to make sure that a is not null even though all users need to have one ip rule so it should never be null
	if(a != NULL)
	{
		//copy the ip mask into a temp sting
		//we do this because strtok will be called on it and we don't want to destroy the real string
		strcpy_s(mIP,32,a->getipmask());
		//call the check function that does the actual compare
		if( checkIP(nIP,mIP) )
		{
			//if the mask matches the ip we return true
			return true;
		}
	}
	else
	{
		//we should never be inthis else but if we are i guess it should be false
		return false;
	}

	//now we loop through all of the remaining ip rules
	while(a->getnext() != NULL)
	{
		//we make another copy of the original ip because it would have been destroyed by the strtok in checkIP
		strcpy_s(nIP,16,IP);
		//move to the next node
		a = a->getnext();
		//copy of the mask because of the strtok
		strcpy_s(mIP,32,a->getipmask());
		//call the good old check again
		if(checkIP(nIP,mIP))
		{
			return true;
		}
	};
	return false;
}

//check to see if a user has enough credits to download a file
//fsize is the size of the file in bytes
//if the amount of credits the user has is more then fsize we return true
//otehrwise we return false
bool authCredits(user *sUser, int fsize)
{
	if(sUser->getcredits()<0)
	{
		return true;
	}
	else
	{
		if(sUser->getcredits()>=fsize)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
}


//check eo see if credits are on for a user
//if the credits are set to less then 0 then they are off
//so in that case we return false, if they are 0 or over then credits are on
bool authCreditsOn(user *sUser)
{
	if(sUser->getcredits()<0)
	{
		return false;
	}
	return true;
}


bool authGroupList(folder *fldr, int grp)
{
	//creating temp group pointer to move though
	group *temp = fldr->getGfirst();
	//looping while the pointer does not equal null
	while(temp != NULL)
	{
		//if the group of the folder is less then or equal to that of the user
		//we return true because the user has read access to the folder
		if(temp->getName() <= grp)
		{
			return true;
		}
		//move to the next folder in the linked list
		temp = temp->getNext();
	};
	//return false if we didn't find any groups we can access in the folder
	return false;

}



bool authGroupListAtLevel(folder *fldr, int grp)
{
	//loop backward will we find the first folder on this level
	//that will be the one that has last set to null
	while(fldr->getLast() !=NULL)
	{
		fldr = fldr->getLast();
	};

	//loop aslong as the folder does not equal null
	//aka go till we reach the end of the linked list of folders
	while(fldr !=NULL)
	{
		//create a temp group pointer to move though	
		group *temp = fldr->getGfirst();
		//while we are not at the end of the linked list	
		while(temp != NULL)
		{
			//if the group of the folder is less then or equal to that of the user
			//we return true because the user has read access to the folder
			if(temp->getName() <= grp)
			{
				return true;
			}
			//move to the next group in the linked list
			temp = temp->getNext();
		};
		//move to the next folder in the linked list
		fldr = fldr->getNext();
	};
	//return false is there was no folder we have access too
	return false;

}








bool authWriteAccess(folder *fldr, int grp)
{


	//loop aslong as the folder does not equal null
	//aka go till we reach the end of the linked list of folders
	while(fldr !=NULL)
	{
		//create a temp group pointer to move though	
		group *temp = fldr->getGfirst();
		//while we are not at the end of the linked list	
		while(temp != NULL)
		{
			//if the group of the folder is less then or equal to that of the user
			//we return true because the user has read access to the folder
			if(temp->getPremissions() >= WRITEACCESS)
			{
				if(temp->getName() <= grp)
				{
					return true;
				}
			}
			//move to the next group in the linked list
			temp = temp->getNext();
		};
		//move to the next folder in the linked list
		fldr = fldr->getUp();
	};
	//return false is there was no folder we have access too
	return false;

}




bool authDeleteAccess(folder *fldr, int grp, char *path, char *name, char *owner)
{
	folder *ofldr = fldr;

	//loop aslong as the folder does not equal null
	//aka go till we reach the end of the linked list of folders
	while(fldr !=NULL)
	{
		//create a temp group pointer to move though	
		group *temp = fldr->getGfirst();
		//while we are not at the end of the linked list	
		while(temp != NULL)
		{
			//if the group of the folder is less then or equal to that of the user
			//we return true because the user has read access to the folder
			if(temp->getPremissions() >= DELETEACCESS)
			{
				//if the group value is less then or equal to that of the user we need to check file ownership
				if(temp->getName() <= grp)
				{

					//create a tmp file pointer to user	
					file *tfile = ofldr->getFfirst();
					//create a flag to be used to tell if it is a user uploaded file ot directory
					bool ufile=false;
					while(tfile != NULL)
					{

						//check to see if the path of the files are a match
						if(!strcmp(tfile->getPath(),path))
						{
							//if the paths match we then check the file names
							if(!strcmp(tfile->getName(),name))
							{
								//if we have a match there we then set the flag for user file to true
								ufile = true;
								//now er check the owner of the groups
								if(!strcmp(tfile->getOwner(),owner))
								{
									//if you are the owner we return true
									return true;
								}
								else
								{
									//if you are not the owner we check to see if you have greater permissions then that of the file
									//you need to have greater permissiosn to delete, not equal
									if(tfile->getGroup() < grp)
									{
										return true;
									}
								}
							}
						}
						//move to the next node
						tfile = tfile->getNext();
					};
					//if the flag was set and we have not returned yet then you can't delete, so false
					if(ufile)
					{
						return false;
					}
					else
					{
						//it is not a user uploaed file so you have rights to delete it
						return true;
					}




				}
			}
			//move to the next group in the linked list
			temp = temp->getNext();
		};
		//move to the next folder in the linked list
		fldr = fldr->getUp();
	};
	//return false is there was no folder we have access too
	return false;

}












//this fuction checks to see if the file is a file that was uploaed by a user
//if it was a file that was uploaed by a user we are gonna return a pointer to that file
file* authUploadedFile(folder *fldr, char *path, char *name)
{
	//create a temp file pointer to use
	file *tfile = fldr->getFfirst();

	//checl to make sure that it is not null because it would be bad to do stuff with a null object
	while(tfile != NULL)
	{
		//make sure the file is a file and not a directory
		if(!(tfile->getDir()))

		{						
			//now we check to see of that paths match
			if(!strcmp(tfile->getPath(),path))
			{
				//assuming the paths match we are now gonna check the names
				if(!strcmp(tfile->getName(),name))
				{
					//the names match so we return the pointer to the file								
					return tfile;

				}
			}
		}
		//move to the next node
		tfile = tfile->getNext();
	};
	//there was no match so in this case we return a null pointer
	return NULL;

}



//this function checks to see if a directory is a directory that was created by a user
//if it is a directory that was created by a user then we return a pointer to it
file* authMadeDir(folder *fldr, char *path, char *name)
{
	//same old creating a temp pointer to use
	file *tfile = fldr->getFfirst();

	//make sure it isnot null and keep going until it is basiclly
	while(tfile != NULL)
	{
		//see if it is a directory not a file
		if(tfile->getDir())
		{
			//see if the path is a match
			if(!strcmp(tfile->getPath(),path))
			{
				//now we see if the name is a match
				if(!strcmp(tfile->getName(),name))
				{
					//if the name and the path are a match we return a pointer to the directory
					return tfile;

				}
			}
		}
		//move to the next node
		tfile = tfile->getNext();
	};
	//return null if we do not have a match
	return NULL;

}

//check to see if a group exists
bool authIsGroup(char *name)
{

	//load the group by name from the xmlfile
	xml_manager *myxml = new xml_manager("group.xml");
	usergroup *mygroup = myxml->loadGroup(name);
	delete myxml;

	//if mygroup is null that means that is could not be found
	if(mygroup==NULL)
	{
		return false;
	}
	else
	{
		//if the group was loaded we delete it and return true
		delete mygroup;
		return true;
	}


}

//this fuctions checks to to see if a group is used by any of the users in the database
bool authGroupUsed(char *groupname)
{
	xml_manager *myxml = new xml_manager("group.xml");
	bool rval = myxml->checkIfGroupUsed(groupname);
	delete myxml;
	return rval;

}