/***********************************************
** crypt.cpp           horseshoeFTPd          **
** Nick Byra                                  **
** 21/03/06                                   **
**                                            **
***********************************************/

#include "crypt.h"
#include "mem_manage.h"



//print out the error message that openssl generates in a human readble format
void tls_error(void) 
{
	//std::cout<<ERR_error_string(ERR_get_error(),NULL)<<"\n";
}



//sets up the SSL_ctx that the server uses
SSL_CTX *setup_server_ctx(void)
{
	SSL_CTX *ctx;

	//making this ssl v2 and v3 compatable
	//possibly add suport for tls later
	ctx = SSL_CTX_new(SSLv23_method());

	//load the certificate file
	if (SSL_CTX_use_certificate_chain_file(ctx, "cert.pem") != 1)
	{
		tls_error();
	}

	if (SSL_CTX_use_PrivateKey_file(ctx, "cert.pem",SSL_FILETYPE_PEM) != 1)
	{
		tls_error();
	}

	//return the pointer to the ssl_ctx that was created
	return ctx;
}

//created the SSL from the bio and the ssl_ctx
//the int c not used anymore, might be removed later, can be set to anything
SSL* setupcnx(BIO *a, SSL_CTX *b,int c)
{
	//client = BIO_pop(acc);
	SSL *ssl;

	//creating a new SSL object from the CTX
	if (!(ssl = SSL_new(b)))
	{
		tls_error();
	}

	//linking the SSL to the bio, the same bio is used for both read and write so both arguments are a
	SSL_set_bio(ssl, a, a);


	//setting the SSL created to the accepting state for use with the call SSL_accept which is called later
	SSL_set_accept_state(ssl);


	//returning the pointer to the newly created ssl object
	return ssl;

}


//setting up the bio useing a socket descriptor
BIO* setupbio(SOCKET sd)
{
	BIO *bio;
	//creating a new socket bio
	bio = BIO_new(BIO_s_socket());
	//linking the bio with the socket
	BIO_set_fd(bio,sd,BIO_NOCLOSE);
	//returning the pointer to the newly created bio
	return bio;
}


int seed_prng()
{
	//seeding the randomness that openssl needs to be secure
	unsigned int rnd;
	while (RAND_status() == 0) 
	{
		rnd = rand();
		RAND_seed(&rnd, (int) sizeof rnd);
	}
	return 0;
}

int tls_init_library() 
{
	//loading the inital openssl librarys 
	SSL_library_init();
	SSL_load_error_strings();
	//seeding the randomness for openssl
	seed_prng();

	//i always return zero because i have no error handling
	return 0;
}




char* getsha1hash(char *ptext, char *buf, int len)
{
	const EVP_MD *m;
	EVP_MD_CTX ctx;
	char *ret;
	unsigned int outlen;

	//this function preps the digests that we are gonna use for the sha1
	OpenSSL_add_all_digests();

	//malloc a buffer to hold the value the functuion returns
	unsigned char *hash = (unsigned char*)mymalloc(20);
	char *tmp = (char*)mymalloc(3);
	//load up the sha1
	EVP_DigestInit(&ctx,EVP_sha1());
	//send it the plain text
	EVP_DigestUpdate(&ctx,ptext,len);
	//send it the destination buffer
	EVP_DigestFinal(&ctx,hash,&outlen);
	//do the clean up to prevernt mem leaks
	EVP_MD_CTX_cleanup(&ctx);

	//convert the thing to ascii
	buf[0] = '\0';
	for(int i=0;i<20;i++)
	{
		sprintf_s(tmp,3,"%02x",hash[i]);
		strcat_s(buf,41,tmp);
	}

	//free up the temp buffers
	free (tmp);
	free (hash);
	//return the pointer to the buffer with the ascii value of the hash
	return buf;

}
