/***********************************************
** ftpd.cpp     horseshoeFTPd                 **
** Nick Byra                                  **
** 21/03/06                                   **
**                                            **
***********************************************/


#include "ftpd.h"
#include "helpfunctions.h"



//main function of the program
//currently takes no arguments
int main(int argc, char* argv[])
{

    mainloop();
	
	}


int mainloop()
{

	// Start Winsock up
	//we do this first because alot of stuff depends on this being started up
	WSAData wsaData;
	int nCode;
	if ((nCode = WSAStartup(MAKEWORD(1, 1), &wsaData)) != 0) {
		std::cerr << "WSAStartup() returned error code " << nCode << "." <<
			std::endl;
		return 255;
	}

	//we now inisitalize the xerces utils at this point
	//in previous versions this was done in the creation of the xml_manager object
	//this lead to problems when there was more then one xml_manager object at any given time
	try {
		XERCES_CPP_NAMESPACE::XMLPlatformUtils::Initialize();
	}
	catch (const XERCES_CPP_NAMESPACE::XMLException& toCatch) {
		char* message = XERCES_CPP_NAMESPACE::XMLString::transcode(toCatch.getMessage());
		//std::cout << "Error during initialization! :\n"<< message << "\n";
		XERCES_CPP_NAMESPACE::XMLString::release(&message);
	}



	//creating ftp_options object
	//loads the settings from the options.xml file
	ftp_options *serverOptions;
	xml_manager *test = new xml_manager("options.xml");
	serverOptions = test->loadOptions();

	//delete the xml parser object
	delete test;



	//load the directory tree from the dir.xml file
	test = new xml_manager("dir.xml");
	folder *tmp = test->loadDir();
	//delete the xml manager object	
	delete test;


	//set the root node of the tree
	serverOptions->setdtree(tmp);

	//force a save of the directory incase it is different then the way it is in the xml file
	//it could be if some user uploaded files or directorys have been deleted
	forceDirSave(serverOptions);



	// Get host and port from server options
	const char* pcHost = serverOptions->getIP();
	int nPort = serverOptions->getPort();

	//create the tcp manager object
	tcp_manager *tcpman = new tcp_manager(serverOptions);

	//create thread id
	DWORD nThreadID;

	//create the worker thread that handles all of the book keeping stuff for the server
	//such as saving files and that sort of thing
	CreateThread(0, 0, WorkerThread, (void*)serverOptions, 0, &nThreadID);



	// Call the main tcp function.
	//this fires up the listener and stuff like that
	int retval = tcpman->DoWinsock(pcHost, nPort);

	// Shut Winsock back down and shut down the xml utils.
	XERCES_CPP_NAMESPACE::XMLPlatformUtils::Terminate();
	WSACleanup();

	//closing the application and returning the retval for no better reason then to return it
	return retval;


}




//this is the worker thread that does all of the book keeping for the server
DWORD WINAPI WorkerThread(LPVOID lpParam ) 
{

	//take the pointer to the serverOptions object that we have been passed
	ftp_options *serverOptions = (ftp_options*)lpParam;

	//do a little loop forever
	//set up a counter to count the times though the loop
	int cnt = 0;
	while(1)
	{
		//sleep for 1000 miliseconds, for those that can;t do math thats 1 second
		Sleep(1000);
		//get the current time, this is used for the bandwidth quotas
		time_t time_now =  time(&time_now);


		//see if the server is set to shut down
		if(serverOptions->getSHUTDOWN())
		{

			//if the server is set to shut down
			//tell all of the sessions to shutdown
			for(int i=0;i<serverOptions->getMaxusers();i++)
			{
				serverOptions->setDIEBITCH(i,true);
			}

			//wait till all of the sessions have closed, check again every second
			while(serverOptions->getCurrentUsers() > 0)
			{
				Sleep(1000);
			};

			//at this point all of the users have been disconected
			//so we save all of the changes to the directory tree
			forceDirSave(serverOptions);

			//now we save all the changed to the server settings
			serverOptions->forcesave();

			//and now we close the program
			//yahoo
			exit(0);

		}


		//do this every x number of seconds, these values are probobly gonna get played with
		if(!(cnt%60))
		{

			//check to see if there are any users currently connected to the ftp
			if(serverOptions->getCurrentUsers())
			{

				//if there are any users connected to the ftp we loop though all of the sids
				for(int i=0;i<serverOptions->getMaxusers();i++)
				{
					//if there is a user connected on that sid
					if(serverOptions->getusers(i) != NULL)
					{
						//we check if the current time is greater then the time that this user is set to reset at
						if(serverOptions->getusers(i)->getresetat() < time_now)
						{
							//need to reset the bandwidth
							//so we set the bandwidth this sessions to zero
							serverOptions->getusers(i)->setbwthissession(0);
							//now we calculate the next reset at value
							//to get that we multiply bandwidth limit type by the number of seconds in a day
							serverOptions->getusers(i)->setresetat(time_now+(serverOptions->getusers(i)->getBWlimitType()*3600*24));
						}
					}
				}
			}
			//now we check to see if the global reset at time is less then the current time
			if(serverOptions->getresetat() < time_now)
			{
				//need to reset the bandwidth
				//so we set the servers bandwidth this session to zero
				serverOptions->setbwthissession(0);
				//to get that we multiply bandwidth limit type by the number of seconds in a day
				serverOptions->setresetat(time_now+(serverOptions->getBWlimitType()*3600*24));
			}


		}








		//do this every x number of seconds, these values are probobly gonna get played with
		if(!(cnt%61))
		{
			//check to see if there are any users connected to the server
			if(serverOptions->getCurrentUsers())
			{
				//if there are users connected to the server
				//load the xml manager with the users confilg file
				xml_manager *xmlm = new xml_manager("user.xml");

				//go though all the sids
				for(int i=0;i<serverOptions->getMaxusers();i++)
				{
					//if a user is connected on that sid
					if(serverOptions->getusers(i) != NULL)
					{
						//we save that user
						xmlm->saveUser(serverOptions->getusers(i));
					}
				}
				//delete the xml manager object
				delete xmlm;

			}
		}

		//do this every x number of seconds, these values are probobly gonna get played with
		if(!(cnt%62))
		{
			//force a save of the server settings
			serverOptions->forcesave();
		}

		//do this every x number of seconds, these values are probobly gonna get played with
		if(!(cnt%63))
		{
			//force a save of the directory tree
			forceDirSave(serverOptions);
		}

		//increment the counter of the number of times thought the loop
		cnt++;
	};



	return 0;
}

