/***********************************************
** helpfunctions.cpp     horseshoeFTPd        **
** Nick Byra                                  **
** 21/03/06                                   **
**                                            **
***********************************************/

#include "stdafx.h"
#include "helpfunctions.h"
#include "consts.h"
#include "xml_manager.h"
#include "crypt.h"
#include "mem_manage.h"


//prettier way of getting the int value of a char
int getIntValue(char c)
{
	return c;
}


//chops of the \r\n at the end of a string
char* fixCR(char *token)
{
	if(token == NULL)
	{
		return NULL;
	}

	int strend = strlen(token)-1;
	if((token[strend]==13) || (token[strend]==10))
	{

		token[strend] = '\0';
	}
	if((token[strend-1]==13) || (token[strend-1]==10))
	{

		token[strend-1] = '\0';
	}
	return token;
}

//wrapper function for send that uses SSL
//useTLS is not used right now, will be needed if AUTH tls is implemented
int tls_send(bool useTLS, SOCKET sd,char *buf, int len, int flags,SSL *tls_cnx)
{
	//checks to see if tls_cnx is null, if not uses ssl
	if(tls_cnx != NULL)
	{
		//writes the data out and assigns the return value to retval
		int retval = SSL_write(tls_cnx, buf, len);

		//check to see if an error has occured
		if(retval <= 0)
		{
			tls_error();
		}

		//return the value of retval as is reguardless if an error has occured
		return retval;

	}
	//if ssl is not being use do a basic send and return the value
	return send(sd,buf,len,flags);

}



//wrapper function for recv that uses SSL
//useTLS is not used right now, will be needed if AUTH tls is implemented
int tls_recv(bool useTLS, SOCKET sd,char *buf, int len, int flags,SSL *tls_cnx)
{

	//if tls_cnx is not null then we do the ssl read
	if(tls_cnx != NULL)
	{
		//reads the SSL and returns the value
		return SSL_read(tls_cnx, buf, len);
	}
	//if not ssl does the normal recv and returns the value
	return recv(sd,buf,len,flags);
}

//this functions starts at the end of a string and finds the first occurance of /
//we assume that the last charater in the string, not including \0 is going to be / so we skip that
int strslash(int bsize,char *str)
{
	for(int i=bsize-2;i>=0;i--)
	{
		if(str[i]=='/')
		{
			return i;
		}

	}
	//return -1 if there was no / found in the string
	return -1;

}


//this changes the values of / in a string to \\
//this is because windows handles their directorys in a less cool way then *nix
char* switchbsfs(int bsize,char *str)
{
	for(int i=0;i<bsize;i++)
	{
		if(str[i]=='/')
		{
			str[i]='\\';
		}

	}
	return str;

}

char* getTF(bool a)
{
	if(a)
	{
		return "true";
	}
	else
	{
		return "false";
	}

}


int countslash(int bsize,char *str)
{
	int cnt = 0;
	for(int i=0;i<bsize;i++)
	{
		if(str[i]=='/')
		{
			cnt++;
		}

	}
	return cnt;

}




//sets all of the indexes in buff up to size = to 0
void clearbuff(char *buff, int size)
{

	for(int i=0;i<size;i++)
	{
		buff[i] = '0';

	}


}

void forceDirSave(ftp_options *serverOptions)
{

	xml_manager *test = new xml_manager();
	test->saveDir(serverOptions->getdtree(ROOTGROUP));
	delete test;

}


bool checkAlphaNumeric(char *line,int max)
{

	if(line == NULL)
	{
		return false;
	}


	if(max)
	{
		if(strlen(line) > max)
		{
			return false;
		}
	}

	for(int i=0;i<strlen(line);i++)
	{
		if( ((line[i] >=48) && (line[i] <=57)) ||  ((line[i] >=65) && (line[i] <=90)) || ((line[i] >=97) && (line[i] <=122)) || (line[i] == 95) )
		{
			//looking good
		}
		else
		{
			return false;
		}

	}

	return true;

}



bool checkNumeric(char *line)
{



	for(int i=0;i<strlen(line);i++)
	{
		if( ((line[i] >=48) && (line[i] <=57)) )
		{
			//looking good
		}
		else
		{
			return false;
		}

	}

	return true;

}


//check the syntax of an ipmask to see if it is correct
bool checkIPsyntax(char *ip)
{

	char *myip = (char*)mymalloc(strlen(ip)+1);
	char *next_token;
	char *token;
	strcpy_s(myip,strlen(ip)+1,ip);
	token = strtok_s(myip,".",&next_token);

	//loop through all 4 sections of the ip
	for(int i=0;i<4;i++)
	{
		//if the token is null it fails because the ip is not long enough
		if(token==NULL)
		{
			free(myip);
			return false;
		}

		//if the token is not of length 1,3,7 it fails because all valid formats are of one of those lengths
		if( (strlen(token) != 1) && (strlen(token) != 3)  && (strlen(token) != 7) )
		{
			free(myip);
			return false;
		}

		//if the length is one then the string must be *
		if(strlen(token) == 1)
		{
			if(strcmp(token,"*"))
			{
				free(myip);
				return false;
			}

		}

		//if the string is of length 3 then it must be a number in the range of 000-255
		if(strlen(token) == 3)
		{
			if(!checkNumeric(token))
			{
				free(myip);
				return false;
			}

			if( (atoi(token) < 0) || (atoi(token) > 255) )
			{
				free(myip);
				return false;
			}

		}

		//if the string is of length 7 if must be a number in the range of 000-255 a '-' 
		//and then another number in the range of 000-255
		if(strlen(token) == 7)
		{

			//make sure there is a - in the middle
			if(token[3] != '-')
			{
				free(myip);
				return false;
			}

			//make the - an end of string so we can check the first part
			token[3] = '\0';

			if(!checkNumeric(token))
			{
				free(myip);
				return false;
			}

			if( (atoi(token) < 0) || (atoi(token) > 255) )
			{
				free(myip);
				return false;
			}

			//shift over the second string to the begining to test that
			token[0] = token[4];
			token[1] = token[5];
			token[2] = token[6];

			if(!checkNumeric(token))
			{
				free(myip);
				return false;
			}

			if( (atoi(token) < 0) || (atoi(token) > 255) )
			{
				free(myip);
				return false;
			}

			//move to the next token
			token = strtok_s(NULL,".",&next_token);

		}

	}

	//if nothing failed then the ip is ok so we return true
	free(myip);
	return true;


}

void getIPstring(unsigned long IP, char *buff)
{
	CCriticalSection m_Critical;
	in_addr a;
	a.S_un.S_addr = IP;
	m_Critical.Lock();
	char *tmp = inet_ntoa(a);
	strcpy_s(buff,16,tmp);
	m_Critical.Unlock();
}



void addDir(folder *cfldr, folder *nfldr)
{

	if(cfldr->getNext() == NULL)
	{
		cfldr->setNext(nfldr);
		nfldr->setUp(cfldr->getUp());
		return;
	}

	while(cfldr->getNext() != NULL)
	{
		cfldr = cfldr->getNext();
	}


	cfldr->setNext(nfldr);
	nfldr->setUp(cfldr->getUp());


}

bool addGroupToDir(folder *fldr, group *grp)
{

	if(fldr->getGfirst() == NULL)
	{
		fldr->setGfirst(grp);
		return true;
	}
	group *tmp = fldr->getGfirst();

	if( (tmp->getName() == grp->getName()) &&(tmp->getPremissions() == grp->getPremissions() ))
	{
		return false;
	}
	while(tmp->getNext() != NULL)
	{
		tmp = tmp->getNext();
		if( (tmp->getName() == grp->getName()) &&(tmp->getPremissions() == grp->getPremissions() ))
		{
			return false;
		}
	};
	tmp->setNext(grp);
	grp->setLast(tmp);

	return true;

}




bool delDir(folder *fldr, char *name)
{


	while(fldr != NULL)
	{
		if(!strcmp(fldr->getName(),name))
		{
			folder *tfldr = fldr->getLast();

			if(tfldr == NULL)
			{
				if(fldr->getNext() == NULL)
				{
					//this is the only folder
					fldr->getUp()->setDown(fldr->getDown());
					delete fldr;
					return true;
				}
				else
				{
					fldr->getUp()->setDown(fldr->getNext());
					fldr->getNext()->setLast(NULL);
					delete fldr;
					return true;

				}

			}
			else
			{
				tfldr->setNext(fldr->getNext());
				fldr->getNext()->setLast(tfldr);
				delete fldr;
				return true;
			}

		}
		fldr = fldr->getNext();
	}

	return false;



}
