/***********************************************
** parser.cpp     horseshoeFTPd               **
** Nick Byra                                  **
** 21/03/06                                   **
**                                            **
***********************************************/


#include "parser.h"
#include "auth.h"
#include <direct.h>
#include "helpfunctions.h"
#include "ls.h"
#include "xml_manager.h"
#include <string.h>
#include <iostream>
#include "ws-util.h"
#include "site.h"
#include "logging.h"
#include "mem_manage.h"

using namespace std;



parser::parser(tcp_manager *a)
{
	tcpman = a;
}

parser::~parser()
{
}


int parser::parse(SOCKET sd, char *line, user **sUser,ftp_options *serverOptions, int sid)
{

	SSL *tls_cnx = serverOptions->getSSL(sid);


	char *token;
	char *next_token;
	char *tmp = (char*)mymalloc(strlen(line)+1);
	//making a copy of the line for the format checker
	//might not need to do this, need to find out if strtok destroyes the line

	strncpy_s(tmp,strlen(line)+1,line,strlen(line));

	//calling the format checker to see if the line is a valid one before the command is parsed
	//returns 0 if bad syntax and -1 if unknown command
	int formaterror = formatChecker(tmp);

	free(tmp);

	//test to see where this fucks up
	if((*sUser)!=NULL)
	{
		(*sUser)->printValues();
	}





	//if bad syntax
	if(formaterror==0)
	{

		tcpman->sendline(sd,501,"Syntax error in parameters or arguments",tls_cnx);
		return 0;
	}

	//if command unknown
	if(formaterror==-1)
	{

		tcpman->sendline(sd,500,"Syntax Error, command unrecognized",tls_cnx);
		return 0;
	}

	//breaking the string up by spaces
	token = strtok_s(line, " \n", &next_token);

	//remove '\n' if there is only one token
	int strend = strlen(token)-1;
	//cout<<"#"<<getIntValue(token[strend])<<"#\n";
	token = fixCR(token);



	//check for each command that server supports
	if(!strcmp(token,"USER"))
	{
		token = strtok_s(NULL, " \n", &next_token);
		token = fixCR(token);

		if(*sUser == NULL)
		{
			*sUser = serverOptions->getUserByName(token);
			if(*sUser == NULL)
			{
				xml_manager *xmlm = new xml_manager("user.xml");
				*sUser = xmlm->loadUser(token);
				delete xmlm;
			}

			if(serverOptions->addUsers(*sUser,sid) != SUCCESSFUL)
			{
				Log(LOGIN,(*sUser)->getname(),UNKNOWNERROR,serverOptions->getremoteIP(sid));
				tcpman->sendline(sd,530,"Logon error",tls_cnx);
				return QUIT;
			}

		}

		if(*sUser == NULL)
		{
			Log(LOGIN,"",USERNOTFOUND,serverOptions->getremoteIP(sid));
			tcpman->sendline(sd,530,"Bad Username",tls_cnx);
			return QUIT;
		}

		if(!authAccountActive(*sUser))
		{
			Log(LOGIN,(*sUser)->getname(),ACCOUNTDISABLED,serverOptions->getremoteIP(sid));
			tcpman->sendline(sd,530,"Account not active",tls_cnx);
			return QUIT;
		}

		//off for testing, turn back on
		if(authUserIP(serverOptions->getremoteIP(sid),*sUser) != AUTHORIZED)
		{
			Log(LOGIN,(*sUser)->getname(),INVALIDIP,serverOptions->getremoteIP(sid));
			tcpman->sendline(sd,530,"Not a valid IP",tls_cnx);
			return QUIT;

		}


		if(authUserSessions(serverOptions, *sUser) != AUTHORIZED)
		{
			Log(LOGIN,(*sUser)->getname(),OVERMAXLOGINS,serverOptions->getremoteIP(sid));
			tcpman->sendline(sd,530,"Over max login sessions",tls_cnx);
			return QUIT;

		}
		serverOptions->setcurrentdir(sid,serverOptions->getdtree((*sUser)->getgroup()));


		if((*sUser)->checkPasswordrequired())
		{
			tcpman->sendline(sd,331,"Username good, Password Required",tls_cnx);
		}
		else
		{
			Log(LOGIN,(*sUser)->getname(),LOGGEDIN,serverOptions->getremoteIP(sid));
			tcpman->sendline(sd,230,"User Logged in",tls_cnx);
		}
		return 0;
		//USER <SP> <username> <CRLF>
	}
	else if(!strcmp(token,"PASS"))
	{

		token = strtok_s(NULL, " \n", &next_token);
		token = fixCR(token);
		char *password = (char*)mymalloc(41);
		getsha1hash(token, password, strlen(token));
		if((*sUser)->checkPassword(password))
		{
			Log(LOGIN,(*sUser)->getname(),LOGGEDIN,serverOptions->getremoteIP(sid));
			tcpman->sendline(sd,230,"User Logged in",tls_cnx);
		}
		else
		{
			Log(LOGIN,(*sUser)->getname(),INVALIDPASSWORD,serverOptions->getremoteIP(sid));
			tcpman->sendline(sd,530,"Password is invalid",tls_cnx);
		}

		free(password);

		return 0;

	}
	else if(!strcmp(token,"ACCT"))
	{
		tcpman->sendline(sd,202,"ACCT not implemented, not needed",tls_cnx);
		return 0;
	}
	else if(!strcmp(token,"CWD"))
	{
		//this is fucked up and needs to be fixed


		//CWD  <SP> <pathname> <CRLF>

		token = strtok_s(NULL,"", &next_token);
		token = fixCR(token);

		if(!strcmp(token,".."))
		{
			CDUP(sd,serverOptions,sid);
			return 0;
		}


		if(strlen(token)==0)
		{
			token[0] = '/';
			token[1] = '\0';
		}
		folder *tmp;
		folder *current;
		char *currentpath;
		current = serverOptions->getcurrentdir(sid);
		char *a = serverOptions->getsubdirs(sid);

		currentpath = (char*)mymalloc((strlen(a)+1)*sizeof(char));
		strcpy_s(currentpath,strlen(a)+1,a);



		//if total path is given
		if((token[0] == '/') && (strlen(token) != 1))
		{
			serverOptions->setsubdirs(sid,"");
			tmp = serverOptions->getdtree(ROOTGROUP);

			token = strtok_s(token, "/", &next_token);
			token = fixCR(token);

			//cout<<"token is:"<<token<<"\n";
		}
		else
		{	
			tmp = serverOptions->getcurrentdir(sid);
			token = strtok_s(token,"/", &next_token);
			token = fixCR(token);
			if(token == NULL)
			{
				if(authGroupList(serverOptions->getdtree(ROOTGROUP)->getDown(),(*sUser)->getgroup()))
				{
					serverOptions->setsubdirs(sid,"");
					serverOptions->setcurrentdir(sid,serverOptions->getdtree((*sUser)->getgroup()));
					tcpman->sendline(sd,250,"CWD command successful",tls_cnx);
					return 0;
				}
				else
				{
					tcpman->sendline(sd,550,"Access denied",tls_cnx);
					return 0;
				}
			}

		}



		do
		{

			int NL = 0;
			if(tmp != NULL)
			{
				tmp = tmp->getDown();
			}
			while(tmp != NULL)
			{

				if(!strcmp(token,tmp->getName()))
				{


					serverOptions->setcurrentdir(sid,tmp);
					NL = 1;
					break;


				}
				tmp = tmp->getNext();

			};

			if(!NL)
			{
				////std::cout<<"the folder is "<<tmp->getName()<<"\n";
				if(authGroupList(serverOptions->getcurrentdir(sid),(*sUser)->getgroup()))
				{
					if(checkdir(serverOptions->gettotalrealpath(sid),token,serverOptions->getcurrentdir(sid),serverOptions->getusers(sid)))
					{
						serverOptions->addsubdir(sid,token);
						//tcpman->sendline(sd,250,"CWD command successful",tls_cnx);
						//return 0;

						NL = 1;
					}
				}
				else
				{
					tcpman->sendline(sd,550,"Access denied",tls_cnx);
					serverOptions->setcurrentdir(sid,current);
					serverOptions->setsubdirs(sid,currentpath);
					free (currentpath);
					return 0;
				}
			}
			if(!NL)
			{
				//not found
				tcpman->sendline(sd,550,"Directory not found",tls_cnx);
				serverOptions->setcurrentdir(sid,current);
				serverOptions->setsubdirs(sid,currentpath);
				free (currentpath);
				return 0;
			}



			token = strtok_s(NULL, "/", &next_token);
			token = fixCR(token);
		}
		while(token != NULL);

		//in that dir already
		tcpman->sendline(sd,250,"CWD command successful",tls_cnx);




		return 0;
	}
	else if(!strcmp(token,"CDUP"))
	{

		CDUP(sd,serverOptions,sid);
		return 0;
	}
	else if(!strcmp(token,"SMNT"))
	{
		tcpman->sendline(sd,202,"SMNT not implemented, not needed",tls_cnx);
		return 0;
	}
	else if(!strcmp(token,"SSCN"))
	{
		//this is a hack to get site to site ssl to work
		//with flashfxp atleast
		tcpman->sendline(sd,200,"SSCN:CLIENT METHOD",tls_cnx);
		return 0;
	}
	else if(!strcmp(token,"QUIT"))
	{

		tcpman->sendline(sd,221,"Goodbye",tls_cnx);
		return QUIT;

	}
	else if(!strcmp(token,"REIN"))
	{
		tcpman->sendline(sd,202,"REIN not implemented, not needed",tls_cnx);
		return 0;
	}
	else if(!strcmp(token,"PORT"))
	{

		char *ip = (char*)calloc(16,sizeof(char));

		for(int i=0;i<3;i++)
		{
			token = strtok_s(NULL, ",", &next_token);
			//token = fixCR(token);
			strcat_s(ip,16,token);
			strcat_s(ip,16,".");
		}
		token = strtok_s(NULL, ",", &next_token);
		//token = fixCR(token);
		strcat_s(ip,16,token);


		if(!(*sUser)->getfxp())
		{
			sockaddr_in b;
			b.sin_addr.S_un.S_addr = serverOptions->getremoteIP(sid);
			char *rIP = inet_ntoa(b.sin_addr);
			if(strcmp(rIP,ip))
			{
				tcpman->sendline(sd,553,"FXP not allowed",tls_cnx);
				return 0;
			}

		}


		int a,b;
		token = strtok_s(NULL, ",", &next_token);
		token = fixCR(token);
		a = atoi(token);
		token = strtok_s(NULL, ",", &next_token);
		token = fixCR(token);
		b = atoi(token);


		SOCKET dsd = tcpman->SetUpConnect(ip, (a*256)+b,sid);
		if(dsd > 0)
		{
			serverOptions->setDataSock(sid,dsd);
			tcpman->sendline(sd,200,"PORT command successful",tls_cnx);
		}
		return 0;
	}
	else if( (!strcmp(token,"PASV")) || (!strcmp(token,"CPSV")) )
	{


		SOCKET psd = tcpman->DoPasv(sd, sid, serverOptions);

		return 0;
	}
	else if(!strcmp(token,"TYPE"))
	{
		//TYPE <SP> <type-code> <CRLF>
		token = strtok_s(NULL, " ", &next_token);
		token = fixCR(token);
		if(!strcmp(token,"A"))
		{
			serverOptions->setmode(sid,TYPE_A);
			tcpman->sendline(sd,200,"Type set to A",tls_cnx);
		}
		else if(!strcmp(token,"E"))
		{
			//serverOptions->setmode(sid,TYPE_E);
			tcpman->sendline(sd,504,"Type E not allowed, use A or I",tls_cnx);
		}
		else if(!strcmp(token,"I"))
		{
			serverOptions->setmode(sid,TYPE_I);
			tcpman->sendline(sd,200,"Type set to I",tls_cnx);
		}
		else if(!strcmp(token,"L"))
		{
			//serverOptions->setmode(sid,TYPE_L);
			tcpman->sendline(sd,504,"Type L not allowed, use A or I",tls_cnx);
		}
		else
		{
			tcpman->sendline(sd,504,"Type not valid, use A or I",tls_cnx);
		}
		return 0;
	}
	else if(!strcmp(token,"STRU"))
	{
		token = strtok_s(NULL, " \n", &next_token);
		token = fixCR(token);

		if(!strcmp(token,"F"))
		{
			tcpman->sendline(sd,200,"STRU set to F",tls_cnx);
		}
		else
		{
			tcpman->sendline(sd,504,"STRU not implemented for that parameter",tls_cnx);
		}


		return 0;
	}
	else if(!strcmp(token,"MODE"))
	{
		token = strtok_s(NULL, " \n", &next_token);
		token = fixCR(token);

		if(!strcmp(token,"S"))
		{
			tcpman->sendline(sd,200,"MODE set to S",tls_cnx);
		}
		else
		{
			tcpman->sendline(sd,504,"MODE not implemented for that parameter",tls_cnx);
		}
		return 0;
	}
	else if(!strcmp(token,"RETR"))
	{

		token = strtok_s(NULL, "", &next_token);
		token = fixCR(token);

		int retval;
		if(checkfile(serverOptions->gettotalrealpath(sid),token))
		{

			if(serverOptions->getDataSock(sid) != NULL)
			{

				int fsize = getfilesize(serverOptions->gettotalrealpath(sid),token);

				if(serverOptions->getusessl())
				{
					SSL_accept(serverOptions->getdataSSL(sid));
				}
				if(authCredits(*sUser,fsize-serverOptions->getresumeval(sid)))
				{
					char tmp[1024];
					if(serverOptions->getmode(sid) == TYPE_I)
					{
						sprintf_s(tmp,1024,"Opening BINARY mode data connection (%i bytes)",fsize);
					}
					else
					{
						sprintf_s(tmp,1024,"Opening ASCII mode data connection (%i bytes)",fsize);
					}
					tcpman->sendline(sd,150, tmp,tls_cnx);
					sprintf_s(tmp,1024,"%s%s",serverOptions->gettotalrealpath(sid),token);
					//gotta malloc the buffer
					char *dwBuffer = (char*)mymalloc(serverOptions->getPacketsize()*sizeof(char));
					retval = tcpman->readFile(sd,serverOptions->getDataSock(sid),serverOptions->getresumeval(sid),tmp,dwBuffer,serverOptions->getPacketsize(),serverOptions->getdataSSL(sid),sid);
					//setting resume value back to zero so the next file operation starts at the begning
				}

				else
				{

					retval = NOTENOUGHCREDITS;	
				}
				serverOptions->setresumeval(sid,0);		
				if(serverOptions->getusessl())
				{
					if(int test = SSL_shutdown(serverOptions->getdataSSL(sid)) != 1)
					{

						//std::cout<<ERR_error_string(ERR_get_error(),NULL)<<"\n";;
					}
					SSL_free(serverOptions->getdataSSL(sid));
				}
				//tls_free_library(NULL,serverOptions->getdataSSL(sid));
				serverOptions->setdataSSL(sid,NULL);


				if (ShutdownConnection(serverOptions->getDataSock(sid)))
				{
					cout << "Connection is down." << endl;
				}

				serverOptions->setDataSock(sid,NULL);
				if(retval == ABORTSENT)
				{
					tcpman->sendline(sd,426, "Transfer aborted",tls_cnx);
					tcpman->sendline(sd,226, "Abort command sucessful",tls_cnx);
				}
				else if(retval == NOTENOUGHCREDITS)
				{
					tcpman->sendline(sd,426, "Not enough credits",tls_cnx);
				}
				else if(retval == OVERBANDWIDTHQUOTA)
				{
					tcpman->sendline(sd,426, "Over bandwidth quota",tls_cnx);
				}
				else if(retval == DATATIMEOUT)
				{
					tcpman->sendline(sd,426, "Data connection timeout",tls_cnx);
				}
				else if(retval < 0)
				{
					tcpman->sendline(sd,426, "Transfer did not complete",tls_cnx);
				}
				else if(retval == FORCINGDISCONNECT)
				{
					tcpman->sendline(sd,426, "Transfer Stoped, server shuting down",tls_cnx);
					return QUIT;
				}
				else
				{
					tcpman->sendline(sd,226, "Transfer Complete",tls_cnx);
				}

			}

		}
		else
		{
			tcpman->sendline(sd,553, "File name not found",tls_cnx);
		}

		return 0;
	}
	else if(!strcmp(token,"STOR"))
	{

		token = strtok_s(NULL, "", &next_token);
		token = fixCR(token);

		serverOptions->setincomingfile(sid,token);

		if(serverOptions->getDataSock(sid) != NULL)
		{

			if(checkfile(serverOptions->gettotalrealpath(sid),token))
			{
				if(!authAppendAccess(serverOptions->getcurrentdir(sid),(*sUser)->getgroup(),serverOptions->getsubdirs(sid),token,(*sUser)->getname()))
				{
					tcpman->sendline(sd,553, "Access denied",tls_cnx);
					return 0;
				}
			}


			if(!authWriteAccess(serverOptions->getcurrentdir(sid), (*sUser)->getgroup()))
			{
				tcpman->sendline(sd,553, "Access denied",tls_cnx);
				return 0;
			}

			if(serverOptions->getusessl())
			{
				SSL_accept(serverOptions->getdataSSL(sid));
			}
			int fsize = getfilesize(serverOptions->gettotalrealpath(sid),token);
			char tmp[1024];
			if(serverOptions->getmode(sid) == TYPE_I)
			{
				sprintf_s(tmp,1024,"Opening BINARY mode data connection for %s",serverOptions->getincomingfile(sid));
			}
			else
			{
				sprintf_s(tmp,1024,"Opening ASCII mode data connection for %s",serverOptions->getincomingfile(sid));
			}

			tcpman->sendline(sd,150, tmp,tls_cnx);
			sprintf_s(tmp,1024,"%s%s",serverOptions->gettotalrealpath(sid),token);
			//malloc a buffer for the data beinf read in
			char *dwBuffer = (char*)mymalloc(serverOptions->getPacketsize()*sizeof(char));
			int retval = tcpman->saveFile(sd,serverOptions->getDataSock(sid),serverOptions->getresumeval(sid),tmp,dwBuffer,serverOptions->getPacketsize(),serverOptions->getdataSSL(sid),sid);
			serverOptions->setresumeval(sid,0);

			if(serverOptions->getusessl())
			{
				if(SSL_shutdown(serverOptions->getdataSSL(sid)) != 1)
				{
					tls_error();
				}
				SSL_free(serverOptions->getdataSSL(sid));
			}

			serverOptions->setdataSSL(sid,NULL);


			if (ShutdownConnection(serverOptions->getDataSock(sid)))
			{
				cout << "Connection is down." << endl;
			}

			serverOptions->setDataSock(sid,NULL);
			if(retval == ABORTSENT)
			{
				tcpman->sendline(sd,426, "Transfer aborted",tls_cnx);
				tcpman->sendline(sd,226, "Abort command sucessful",tls_cnx);
			}
			else if(retval == OUTOFDISKSPACE)
			{
				tcpman->sendline(sd,426, "Out of disk space",tls_cnx);
			}
			else if(retval == DATATIMEOUT)
			{
				tcpman->sendline(sd,426, "Data connection timeout",tls_cnx);
			}
			else if(retval == UNKNOWNERROR)
			{
				tcpman->sendline(sd,426, "An unknown error has occured",tls_cnx);
			}
			else if(retval == FORCINGDISCONNECT)
			{
				tcpman->sendline(sd,426, "Transfer Stoped, server shuting down",tls_cnx);
				return QUIT;
			}
			else
			{
				tcpman->sendline(sd,226, "Transfer Complete",tls_cnx);
			}

		}


		return 0;
	}
	else if(!strcmp(token,"STOU"))
	{

		tcpman->sendline(sd,202,"STOU not implemented, name your own files",tls_cnx);

		return 0;
	}
	else if(!strcmp(token,"APPE"))
	{
		//APPE <SP> <pathname> <CRLF>
		return 0;
	}
	else if(!strcmp(token,"ALLO"))
	{
		// ALLO <SP> <decimal-integer>
		//[<SP> R <SP> <decimal-integer>] <CRLF>
		return 0;
	}
	else if(!strcmp(token,"REST"))
	{
		//REST <SP> <marker> <CRLF>

		token = strtok_s(NULL, "", &next_token);
		token = fixCR(token);
		serverOptions->setresumeval(sid,atoi(token));
		tmp[25];
		sprintf_s(tmp,25,"Restarting at %s",token);
		tcpman->sendline(sd,350, tmp,tls_cnx);

		return 0;
	}
	else if(!strcmp(token,"RNFR"))
	{
		token = strtok_s(NULL, "", &next_token);
		token = fixCR(token);

		tmp[255];
		sprintf_s(tmp,255,"%s%s",serverOptions->gettotalrealpath(sid),token);

		if(checkfile(serverOptions->gettotalrealpath(sid),token))
		{
			if(authAppendAccess(serverOptions->getcurrentdir(sid),(*sUser)->getgroup(),serverOptions->getsubdirs(sid),token,(*sUser)->getname()))
			{
				tcpman->sendline(sd,350,"File exists",tls_cnx);
				serverOptions->setincomingfile(sid,token);		
				return 0;
			}
			else
			{
				Log(TRANSFER, serverOptions->getusers(sid)->getname(), RENAME,tmp, ACCESSDENIED, serverOptions->getremoteIP(sid));
				tcpman->sendline(sd,550,"Access denied",tls_cnx);
				return 0;
			}


		}
		else if(checkdir(serverOptions->gettotalrealpath(sid),token,serverOptions->getcurrentdir(sid),serverOptions->getusers(sid)))
		{
			tcpman->sendline(sd,350,"Directory exists",tls_cnx);
			serverOptions->setincomingfile(sid,token);
			return 0;
		}
		else
		{
			Log(TRANSFER, serverOptions->getusers(sid)->getname(), RENAME,tmp, NOTFOUND, serverOptions->getremoteIP(sid));
			tcpman->sendline(sd,550,"No such file or directory",tls_cnx);
			return 0;
		}
		return 0;
	}
	else if(!strcmp(token,"RNTO"))
	{

		token = strtok_s(NULL, "", &next_token);
		token = fixCR(token);
		int rval = 1;

		if(checkVdir(serverOptions->getincomingfile(sid), serverOptions->getcurrentdir(sid)))
		{
			folder *fldr = serverOptions->getcurrentdir(sid)->getDown();

			while(fldr != NULL)
			{

				if(!strcmp(serverOptions->getincomingfile(sid),fldr->getName()))
				{
					fldr->setName(token);
					rval = 0;
					break;
				}
				fldr = fldr->getNext();
			};
		}
		else
		{

			int tempstr_len = strlen(serverOptions->getincomingfile(sid)) + strlen(serverOptions->gettotalrealpath(sid)) +2;
			char *tempstr = (char*)mymalloc(tempstr_len*sizeof(char));
			int tempstr2_len = strlen(serverOptions->gettotalrealpath(sid)) + strlen(token) +2;
			char *tempstr2 = (char*)mymalloc(tempstr_len*sizeof(char));
			sprintf_s(tempstr,tempstr_len,"%s%s",serverOptions->gettotalrealpath(sid),serverOptions->getincomingfile(sid));
			sprintf_s(tempstr2,tempstr2_len,"%s%s",serverOptions->gettotalrealpath(sid),token);
			rval = rename(tempstr,tempstr2);

			//check to see if there is a <file> that we need to chage the name of
			file *tfile = serverOptions->getcurrentdir(sid)->getFfirst();

			while(tfile != NULL)
			{

				if(!strcmp(tfile->getPath(),serverOptions->getsubdirs(sid)))
				{

					if(!strcmp(tfile->getName(),serverOptions->getincomingfile(sid)))
					{
						tfile->setName(token);
						forceDirSave(serverOptions);
						break;
					}
				}
				tfile = tfile->getNext();
			};



			free(tempstr);
			free(tempstr2);

		}

		if(rval == 0)
		{

			tcpman->sendline(sd,250,"RNTO command succesful",tls_cnx);
		}
		else
		{
			tcpman->sendline(sd,550,"RNTO failed",tls_cnx);
		}




		return 0;
	}
	else if(!strcmp(token,"ABOR"))
	{
		//ABOR <CRLF>
		return 0;
	}
	else if(!strcmp(token,"DELE"))
	{
		token = strtok_s(NULL, "", &next_token);
		token = fixCR(token);
		char tmp[255];
		sprintf_s(tmp,255,"%s/%s",serverOptions->gettotalrealpath(sid),token);
		if(authDeleteAccess(serverOptions->getcurrentdir(sid), (*sUser)->getgroup(),serverOptions->getsubdirs(sid),token,(*sUser)->getname()))
		{

			int fsize = getfilesize(serverOptions->gettotalrealpath(sid),token);
			if( remove( tmp ) == -1 )
			{
				Log(TRANSFER, serverOptions->getusers(sid)->getname(), DELETE,tmp, ERROR, serverOptions->getremoteIP(sid));
				tcpman->sendline(sd,550,"Error deleting file",tls_cnx);
				return 0;
			}
			else
			{
				Log(TRANSFER, serverOptions->getusers(sid)->getname(), DELETE,tmp, COMPLETE, serverOptions->getremoteIP(sid));

				//check to see if the file deleted was one uploaded by a user
				file *tfile = authUploadedFile(serverOptions->getcurrentdir(sid), serverOptions->getsubdirs(sid), token);
				//if it was uploaded by a user
				if(tfile != NULL)
				{
					//check to see if user object is already out there
					user *tuser = serverOptions->getUserByName(tfile->getOwner());
					//if the user object is not already out there
					if(tuser == NULL)
					{
						//load the xml parser
						xml_manager *xmlm = new xml_manager("user.xml");
						//load the user
						tuser = xmlm->loadUser(tfile->getOwner());

						delete xmlm;
						//make sure the user still exists
						if(tuser!=NULL)
						{
							//decrease users credits
							tuser->decCredits(fsize);
						}
						//delete the user becase we are done with him
						delete tuser;
					}
					else
					{
						//decrease users credits
						tuser->decCredits(fsize);
					}
					//delete the file from the mem tree
					if(tfile->getLast() != NULL)
					{
						tfile->getLast()->setNext(tfile->getNext());
					}
					else
					{
						serverOptions->getcurrentdir(sid)->setFfirst(tfile->getNext());
					}
					//setting the value of last on the node infront of this one to the node behind this one
					//aka removing the node from the list
					if(tfile->getNext() != NULL)
					{
						tfile->getNext()->setLast(tfile->getLast());
					}

					//deleteing the node to the file that does not exist anymore

					delete tfile;
					//forceDirSave(serverOptions);


				}



				tcpman->sendline(sd,250,"DELE command successful",tls_cnx);
				return 0;
			}

		}
		Log(TRANSFER, serverOptions->getusers(sid)->getname(), DELETE,tmp, ACCESSDENIED, serverOptions->getremoteIP(sid));
		tcpman->sendline(sd,550,"Access Denied",tls_cnx);

		return 0;
	}
	else if(!strcmp(token,"RMD"))
	{

		token = strtok_s(NULL, "", &next_token);
		token = fixCR(token);

		char *new_token;
		char *nline;
		int err = fullPath(token, &new_token, &nline, serverOptions, sid);

		if(err == DIRECTORYNOTFOUND)
		{
			Log(TRANSFER, serverOptions->getusers(sid)->getname(), DELETE,"", NOTFOUND, serverOptions->getremoteIP(sid));
			tcpman->sendline(sd,550, "Path not found",tls_cnx);
			free (nline);
			return 0;
		}




		char tmp[255];
		sprintf_s(tmp,255,"%s%s",serverOptions->gettotalrealpath(sid),new_token);
		if(authDeleteAccess(serverOptions->getcurrentdir(sid), (*sUser)->getgroup(),serverOptions->getsubdirs(sid),new_token,(*sUser)->getname()))
		{


			if( _rmdir( tmp ) != 0 )
			{
				Log(TRANSFER, serverOptions->getusers(sid)->getname(), DELETE,tmp, ERROR, serverOptions->getremoteIP(sid));
				tcpman->sendline(sd,550,"Error deleting directory",tls_cnx);
				free (nline);
				return 0;
			}
			else
			{

				Log(TRANSFER, serverOptions->getusers(sid)->getname(), DELETE,tmp, COMPLETE, serverOptions->getremoteIP(sid));
				file *tfile = authMadeDir(serverOptions->getcurrentdir(sid), serverOptions->getsubdirs(sid), new_token);
				//if it was uploaded by a user
				if(tfile != NULL)
				{

					//delete the file from the mem tree
					if(tfile->getLast() != NULL)
					{
						tfile->getLast()->setNext(tfile->getNext());
					}
					else
					{
						serverOptions->getcurrentdir(sid)->setFfirst(tfile->getNext());
					}
					//setting the value of last on the node infront of this one to the node behind this one
					//aka removing the node from the list
					if(tfile->getNext() != NULL)
					{
						tfile->getNext()->setLast(tfile->getLast());
					}

					//deleteing the node to the file that does not exist anymore

					delete tfile;
					//forceDirSave(serverOptions);

				}


				tcpman->sendline(sd,250,"RMD command successful",tls_cnx);
				free (nline);
				return 0;
			}

		}
		Log(TRANSFER, serverOptions->getusers(sid)->getname(), DELETE,tmp, ACCESSDENIED, serverOptions->getremoteIP(sid));
		tcpman->sendline(sd,550,"Access Denied",tls_cnx);
		free (nline);
		return 0;
	}
	else if(!strcmp(token,"MKD"))
	{
		//MKD  <SP> <pathname> <CRLF>
		token = strtok_s(NULL, "", &next_token);
		token = fixCR(token);

		char *new_token;
		char *nline;
		int err = fullPath(token, &new_token, &nline, serverOptions, sid);

		if(err == DIRECTORYNOTFOUND)
		{
			tcpman->sendline(sd,550, "Path not found",tls_cnx);
			return 0;
		}

		if(err == ACCESSDENIED)
		{
			tcpman->sendline(sd,550, "Access denied",tls_cnx);
			return 0;
		}


		if(!authWriteAccess(serverOptions->getcurrentdir(sid), (*sUser)->getgroup()))
		{
			tcpman->sendline(sd,550, "Access denied",tls_cnx);
			free (nline);
			return 0;
		}



		int tempstr_len = (strlen(serverOptions->gettotalrealpath(sid))+strlen(new_token)+1+21);
		char *tempstr = (char*)mymalloc(tempstr_len*sizeof(char));
		sprintf_s(tempstr,tempstr_len,"%s%s",serverOptions->gettotalrealpath(sid),new_token);
		//std::cout<<tempstr<<"\n";

		int rval = _mkdir(tempstr);

		if(rval == 0)
		{
			file *nfile = new file();
			nfile->setOwner((*sUser)->getname());
			nfile->setGroup((*sUser)->getgroup());
			nfile->setPath(serverOptions->getsubdirs(sid));
			nfile->setName(new_token);
			nfile->setDir(true);



			file *tfile = serverOptions->getcurrentdir(sid)->getFfirst();
			if(tfile != NULL)
			{
				while(tfile->getNext() != NULL)
				{
					tfile = tfile->getNext();
				};
				tfile->setNext(nfile);
				nfile->setLast(tfile);
			}
			else
			{
				serverOptions->getcurrentdir(sid)->setFfirst(nfile);
			}

			forceDirSave(serverOptions);	



			sprintf_s(tempstr,tempstr_len,"\"%s\" directory created",new_token);
			tcpman->sendline(sd,257,tempstr,tls_cnx);
		}
		else
		{
			sprintf_s(tempstr,tempstr_len,"\"%s\" failed to create",new_token);
			tcpman->sendline(sd,550,tempstr,tls_cnx);
		}
		free (tempstr);
		free (nline);
		return 0;
	}
	else if( (!strcmp(token,"PWD")) || (!strcmp(token,"XPWD")))
	{
		//PWD  <CRLF>
		char tmp[255];
		sprintf_s(tmp,255,"\"%s\" is the current directory",serverOptions->gettotalpath(sid));

		tcpman->sendline(sd,257,tmp,tls_cnx);
		return 0;
	}
	else if(!strcmp(token,"LIST"))
	{

		doLIST(sd,serverOptions,sid);

		return 0;
	}
	else if(!strcmp(token,"NLST"))
	{
		token = strtok_s(NULL,"", &next_token);
		token = fixCR(token);
		//cout<<"the strlen of token is "<<strlen(token)<<"\n";
		folder *tmp;
		folder *current;
		char *currentpath;

		if(token==NULL)
		{
			doLIST(sd,serverOptions,sid);
			return 0;
		}
		else
		{



			current = serverOptions->getcurrentdir(sid);
			char *a = serverOptions->getsubdirs(sid);
			currentpath = (char*)mymalloc((strlen(a)+1)*sizeof(char));
			strcpy_s(currentpath,strlen(a)+1,a);


			//if total path is given
			if((token[0] == '/') && (strlen(token) != 1))
			{
				tmp = serverOptions->getdtree(ROOTGROUP);
				serverOptions->setsubdirs(sid,"");


				token = strtok_s(token, "/", &next_token);
				token = fixCR(token);

				//cout<<"token is:"<<token<<"\n";
			}

			else
			{	
				tmp = serverOptions->getcurrentdir(sid);
				token = strtok_s(token,"/", &next_token);
				token = fixCR(token);
				if(token == NULL)
				{

					if(authGroupList(serverOptions->getdtree(ROOTGROUP),(*sUser)->getgroup()))
					{
						serverOptions->setsubdirs(sid,"");
						serverOptions->setcurrentdir(sid,serverOptions->getdtree((*sUser)->getgroup()));
						doLIST(sd,serverOptions,sid);
						serverOptions->setcurrentdir(sid,current);
						serverOptions->setsubdirs(sid,currentpath);
						free (currentpath);
						return 0;
					}
					else
					{
						tcpman->sendline(sd,550,"Access denied",tls_cnx);
						return 0;
					}
				}

			}


			//cout<<"argument is "<<token<<" a length "<<strlen(token)<<"\n";
			//if(!strcmp(token,serverOptions->getcurrentdir(sid)->getName()))
			if(false)
			{
				//in that dir already
				doLIST(sd,serverOptions,sid);
				serverOptions->setcurrentdir(sid,current);
				serverOptions->setsubdirs(sid,currentpath);
				free (currentpath);
				return 0;
			}
			else
			{

				do
				{
					//cout<<"argument is "<<token<<" a length "<<strlen(token)<<"\n";
					int NL = 0;
					if(tmp != NULL)
					{
						tmp = tmp->getDown();
					}
					while(tmp != NULL)
					{
						if(!strcmp(token,tmp->getName()))
						{
							//serverOptions->setcurrentdir(sid,tmp);
							//tcpman->sendline(sd,250,"CWD command successful",tls_cnx);
							//return 0;
							serverOptions->setcurrentdir(sid,tmp);
							NL = 1;
							break;
						}
						tmp = tmp->getNext();

					};
					//cout<<"NL is "<<NL<<"\n";
					if(!NL)
					{
						if(authGroupList(serverOptions->getcurrentdir(sid),(*sUser)->getgroup()))
						{
							if(checkdir(serverOptions->gettotalrealpath(sid),token,serverOptions->getcurrentdir(sid),serverOptions->getusers(sid)))
							{
								serverOptions->addsubdir(sid,token);
								//tcpman->sendline(sd,250,"CWD command successful",tls_cnx);
								//return 0;

								NL = 1;
							}
						}
						else
						{
							tcpman->sendline(sd,550,"Access denied",tls_cnx);
							serverOptions->setcurrentdir(sid,current);
							serverOptions->setsubdirs(sid,currentpath);
							free (currentpath);
							return 0;
						}
					}
					if(!NL)
					{
						//not found
						tcpman->sendline(sd,550,"Directory not found",tls_cnx);
						serverOptions->setcurrentdir(sid,current);
						serverOptions->setsubdirs(sid,currentpath);
						free (currentpath);
						return 0;
					}



					token = strtok_s(NULL, "/", &next_token);
					token = fixCR(token);
				}
				while(token != NULL);
			}
			doLIST(sd,serverOptions,sid);
			serverOptions->setcurrentdir(sid,current);
			serverOptions->setsubdirs(sid,currentpath);
			free (currentpath);
			return 0;
		}	
		return 0;



	}
	else if(!strcmp(token,"SITE"))
	{
		token = strtok_s(NULL, "", &next_token);
		token = fixCR(token);
		char *sitecmds = (char*)mymalloc(strlen(token)+1);
		strcpy_s(sitecmds,strlen(token)+1,token);
		int rval = siteParser(sitecmds,sid,serverOptions);
		free (sitecmds);

		if(rval == ACCESSDENIED)
		{
			tcpman->sendline(sd,500,"Access Denied",tls_cnx);
		}

		if(rval == SYNTAXINVALID)
		{
			tcpman->sendline(sd,500,"Syntax is invalid",tls_cnx);
		}

		if(rval == EDITGROUP + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Setting changed",tls_cnx);
		}

		if(rval == EDITUSER + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Setting changed",tls_cnx);
		}

		if(rval == ADDUSER + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"User added",tls_cnx);
		}

		if(rval == ADDUSER + INVALIDGROUP)
		{
			tcpman->sendline(sd,500,"Group does not exist",tls_cnx);
		}

		if(rval == ADDGROUP + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Group added",tls_cnx);
		}

		if(rval == DELGROUP + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Group deleted",tls_cnx);
		}


		if(rval == DELGROUP + GROUPINUSE)
		{
			tcpman->sendline(sd,500,"Group connected to a user, can't delete",tls_cnx);
		}


		if(rval == DELUSER + USERNOTFOUND)
		{
			tcpman->sendline(sd,200,"User not found",tls_cnx);
		}

		if(rval == DELGROUP + USERNOTFOUND)
		{
			tcpman->sendline(sd,200,"Group not found",tls_cnx);
		}


		if(rval == DELUSER + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"User deleted",tls_cnx);
		}


		if(rval == EDITUSER + ADDIPFAILED)
		{
			tcpman->sendline(sd,500,"Add IP failed",tls_cnx);
		}

		if(rval == EDITUSER + DELIPFAILED)
		{
			tcpman->sendline(sd,500,"Delete IP failed",tls_cnx);
		}

		if(rval == KILLUSER + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"User killed",tls_cnx);
		}

		if(rval == SERVERSHUTDOWN + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Server shutting down",tls_cnx);
		}

		if(rval == KILLUSER + USERNOTFOUND)
		{
			tcpman->sendline(sd,500,"User not online",tls_cnx);
		}

		if(rval == ADDDIR + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Directory added",tls_cnx);
		}

		if(rval == EDITDIR + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Directory modified",tls_cnx);
		}

		if(rval == DELDIR + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Directory deleted",tls_cnx);
		}

		if(rval == ADDDIRGROUP + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Directory group added",tls_cnx);
		}


		if(rval == SETBWLIMITTYPE + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Bandwidth limit type set",tls_cnx);
		}
		if(rval == SETNAME + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Server name set",tls_cnx);
		}
		if(rval == SSLON + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"SSL on",tls_cnx);
		}
		if(rval == SSLOFF + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"SSL off",tls_cnx);
		}
		if(rval == SETDATAPORTH + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Data port range high set",tls_cnx);
		}
		if(rval == SETDATAPORTL + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Data port range low set",tls_cnx);
		}
		if(rval == SETFREEFILESIZE + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Server free file size set",tls_cnx);
		}
		if(rval == SETTIMEOUT + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Idle timeout set",tls_cnx);
		}
		if(rval == SETCONNECTIONSPERIP + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Max connection per IP set",tls_cnx);
		}
		if(rval == SETMAXUSERS + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Max users set",tls_cnx);
		}
		if(rval == SETPACKETSIZE + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Packetsize set",tls_cnx);
		}
		if(rval == SETPORT + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Server port set",tls_cnx);
		}
		if(rval == SETBWLIMIT + SUCCESSFUL)
		{
			tcpman->sendline(sd,200,"Server bandwidth limit set",tls_cnx);
		}


		if(rval == ADDDIRGROUP + ALREADYEXISTS)
		{
			tcpman->sendline(sd,500,"Directory group already exists",tls_cnx);
		}

		if(rval == DELDIR + DIRNOTFOUND)
		{
			tcpman->sendline(sd,500,"Directory not found",tls_cnx);
		}

		if(rval == ADDDIR + DIRNOTFOUND)
		{
			tcpman->sendline(sd,500,"Directory not found",tls_cnx);
		}

		if(rval == DELUSER + USERONLINE)
		{
			tcpman->sendline(sd,500,"User is online",tls_cnx);
		}
		if(rval == EDITDIR + DIRNOTFOUND)
		{
			tcpman->sendline(sd,500,"Directory not found",tls_cnx);
		}

		return 0;
	}
	else if(!strcmp(token,"SYST"))
	{
		//SYST <CRLF>

		tcpman->sendline(sd,215,"UNIX Type: L8",tls_cnx);
		return 0;

	}
	else if(!strcmp(token,"STAT"))
	{
		tmp[255];
		sprintf_s(tmp,255,"211-Status for user %s",(*sUser)->getname());	
		tcpman->sendline(sd,-1,tmp,tls_cnx);
		sprintf_s(tmp,255,"Currently logged on %i time(s)",serverOptions->getNumConnections((*sUser)));
		tcpman->sendline(sd,0,tmp,tls_cnx);

		sprintf_s(tmp,255,"Downloaded %i KB",(*sUser)->getdownloaded()/1000);
		tcpman->sendline(sd,0,tmp,tls_cnx);
		sprintf_s(tmp,255,"Uploaded %i KB",(*sUser)->getuploaded()/1000);
		tcpman->sendline(sd,0,tmp,tls_cnx);
		if(authCreditsOn((*sUser)))
		{
			sprintf_s(tmp,255,"credits %i",(*sUser)->getcredits());
			tcpman->sendline(sd,0,tmp,tls_cnx);
		}

		if((*sUser)->getBWlimitType())
		{
			sprintf_s(tmp,255,"Bandwidth left on quota %i KB",((*sUser)->getBWlimit()-(*sUser)->getbwthissession())/1000);
			tcpman->sendline(sd,0,tmp,tls_cnx);
		}

		tcpman->sendline(sd,211,"End of status",tls_cnx);

		return 0;
	}
	else if(!strcmp(token,"HELP"))
	{

		tcpman->sendline(sd,-1,"214- The following commands are recognized (* => unimplemented)",tls_cnx);
		tcpman->sendline(sd,0," USER    PORT    RETR   *ALLO    DELE    SITE    XMKD    CDUP    FEAT",tls_cnx);
		tcpman->sendline(sd,0," PASS    PASV    STOR    REST    CWD     STAT    RMD     XCUP    OPTS",tls_cnx);
		tcpman->sendline(sd,0,"*ACCT    TYPE    APPE    RNFR    XCWD    HELP    XRMD    STOU",tls_cnx);
		tcpman->sendline(sd,0," REIN    STRU    SMNT    RNTO    LIST    NOOP    PWD     SIZE",tls_cnx);
		tcpman->sendline(sd,0," QUIT    MODE    SYST    ABOR    NLST    MKD     XPWD    MDTM",tls_cnx);
		tcpman->sendline(sd,214,"End of HELP",tls_cnx);



		return 0;
	}
	else if(!strcmp(token,"NOOP"))
	{
		tcpman->sendline(sd,200,"OK",tls_cnx);
		return 0;
	}
	else if(!strcmp(token,"FEAT"))
	{
		// FEAT <CRLF>
		//list extended features that ftpd supports
		//cout<<"feat\n";
		tcpman->sendline(sd,-1,"211-Extentions supported:",tls_cnx);
		tcpman->sendline(sd,0,"SIZE",tls_cnx);
		tcpman->sendline(sd,0,"SITE",tls_cnx);
		tcpman->sendline(sd,211,"End",tls_cnx);
		return 0;
	}
	else if(!strcmp(token,"PBSZ"))
	{
		//cout<<"PBSZ command\n";
		tcpman->sendline(sd,200,"Command ok",tls_cnx);
		return 0;
	}
	else if(!strcmp(token,"PROT"))
	{
		//cout<<"PROT command\n";
		tcpman->sendline(sd,200,"Command ok",tls_cnx);
		return 0;
	}
	else if(!strcmp(token,"SIZE"))
	{
		//cout<<"PROT command\n";
		token = strtok_s(NULL, "", &next_token);
		token = fixCR(token);
		if(!checkfile(serverOptions->gettotalrealpath(sid),token))
		{
			tcpman->sendline(sd,553, "File Does not exists",tls_cnx);
			return 0;

		}
		int fsize = getfilesize(serverOptions->gettotalrealpath(sid),token);
		char fs[20];
		_itoa_s(fsize,fs,20,10);


		tcpman->sendline(sd,213,fs,tls_cnx);
		return 0;
	}
	else
	{
		//cout<<"you should never be here\n";

		return -1;

	}

	return 0;

}












int parser::formatChecker(char *line)
{
	//cout<<"checker called on "<<line<<"\n";
	char *token;
	char *next_token;
	const int OK = 1;
	const int BAD_SYNTAX = 0;

	token = strtok_s(line, " \n", &next_token);

	//remove '\n' if there is only one token
	int strend = strlen(token)-1;
	//cout<<"#"<<getIntValue(token[strend])<<"#\n";
	token = fixCR(token);

	//cout<<"first token = "<<token<<" length = "<<strlen(token)<<"\n";

	if(!strcmp(token,"USER"))
	{
		return countArgs(token,2,next_token);
	}
	else if(!strcmp(token,"PASS"))
	{
		return countArgs(token,2,next_token);
	}
	else if(!strcmp(token,"ACCT"))
	{
		return countArgs(token,2,next_token);
	}
	else if(!strcmp(token,"CWD"))
	{
		//return countArgs(token,2,next_token);
		return true;
	}
	else if(!strcmp(token,"CDUP"))
	{
		return countArgs(token,1,next_token);
	}
	else if(!strcmp(token,"SMNT"))
	{
		//i'll get to you
		return OK;
	}
	else if(!strcmp(token,"QUIT"))
	{
		return countArgs(token,1,next_token);
	}
	else if(!strcmp(token,"REIN"))
	{
		//i'll get to you later
		return OK;
	}
	else if(!strcmp(token,"PORT"))
	{


		for(int i=0;i<6;i++)
		{			
			token = strtok_s(NULL," ,\n", &next_token);

			if(token != NULL)
			{
				return OK;
			}
			else
			{
				return BAD_SYNTAX;
			}

		}
		token = strtok_s(NULL," ,\n", &next_token);

		if(token == NULL)
		{
			return OK;
		}
		else
		{
			return BAD_SYNTAX;
		}


	}
	else if(!strcmp(token,"PASV"))
	{
		return countArgs(token,1,next_token);

	}
	else if(!strcmp(token,"TYPE"))
	{
		return countArgs(token,2,next_token);
	}
	else if(!strcmp(token,"STRU"))
	{
		return countArgs(token,2,next_token);
	}
	else if(!strcmp(token,"MODE"))
	{
		return countArgs(token,2,next_token);

	}
	else if(!strcmp(token,"RETR"))
	{
		//return countArgs(token,2,next_token);
		return OK;

	}
	else if(!strcmp(token,"STOR"))
	{
		//hack to make this work for now, fix this to deal with spaces
		//return countArgs(token,2,next_token);
		return OK;
	}
	else if(!strcmp(token,"STOU"))
	{
		return countArgs(token,2,next_token);

	}
	else if(!strcmp(token,"APPE"))
	{
		return countArgs(token,2,next_token);

	}
	else if(!strcmp(token,"ALLO"))
	{

		return countArgs(token,2,next_token);

	}
	else if(!strcmp(token,"REST"))
	{
		return countArgs(token,2,next_token);

	}
	else if(!strcmp(token,"RNFR"))
	{
		//return countArgs(token,2,next_token);
		return OK;
	}
	else if(!strcmp(token,"RNTO"))
	{
		//return countArgs(token,2,next_token);
		return OK;
	}
	else if(!strcmp(token,"ABOR"))
	{
		return countArgs(token,1,next_token);

	}
	else if(!strcmp(token,"DELE"))
	{
		return OK;

	}
	else if(!strcmp(token,"RMD"))
	{
		return OK;

	}
	else if(!strcmp(token,"MKD"))
	{
		return OK;
	}
	else if( (!strcmp(token,"PWD")) || (!strcmp(token,"XPWD")))
	{
		return countArgs(token,1,next_token);
	}
	else if(!strcmp(token,"LIST"))
	{
		//later
		return OK;

	}
	else if(!strcmp(token,"NLST"))
	{
		return countArgs(token,2,next_token);

	}
	else if(!strcmp(token,"SITE"))
	{
		//later
		return OK;
	}
	else if(!strcmp(token,"SYST"))
	{
		return countArgs(token,1,next_token);

	}
	else if(!strcmp(token,"STAT"))
	{
		return countArgs(token,1,next_token);
	}
	else if(!strcmp(token,"HELP"))
	{
		//later
		return OK;

	}
	else if(!strcmp(token,"NOOP"))
	{
		return countArgs(token,1,next_token);

	}
	else if(!strcmp(token,"FEAT"))
	{
		return countArgs(token,1,next_token);
	}
	else if(!strcmp(token,"PBSZ"))
	{
		return countArgs(token,2,next_token);
	}
	else if(!strcmp(token,"PROT"))
	{
		return countArgs(token,2,next_token);
	}
	else if(!strcmp(token,"SSCN"))
	{
		return OK;
	}
	else if(!strcmp(token,"CPSV"))
	{
		return OK;
	}
	else if(!strcmp(token,"SIZE"))
	{
		return countArgs(token,2,next_token);
	}
	else
	{


		return -1;

	}

	return 1;

}


int parser::countArgs(char* token, int args, char *next_token)
{
	const int OK = 1;
	const int BAD_SYNTAX = 0;	
	for(int i=0;i<args;i++)
	{
		token = strtok_s(NULL, " ", &next_token);


	}
	if(token == NULL)
	{

		return OK;
	}
	else
	{

		return BAD_SYNTAX;
	}


}


void parser::doLIST(SOCKET sd,ftp_options *serverOptions, int sid)
{
	SSL *tls_cnx = serverOptions->getSSL(sid);

	if(serverOptions->getusessl())
	{
		SSL_accept(serverOptions->getdataSSL(sid));
	}
	if(serverOptions->getmode(sid) == TYPE_I)
	{
		tcpman->sendline(sd,150, "Opening BINARY mode data connection for /bin/ls",tls_cnx);
	}
	else
	{
		tcpman->sendline(sd,150, "Opening ASCII mode data connection for /bin/ls",tls_cnx);
	}

	char *tmp = serverOptions->gettotalrealpath(sid);
	scan(serverOptions->getDataSock(sid),tmp,"*",tcpman,serverOptions->getdataSSL(sid),serverOptions->getcurrentdir(sid),sid);
	if(serverOptions->getusessl())
	{
		if(SSL_shutdown(serverOptions->getdataSSL(sid)) != 1)
		{
			//std::cout<<"this is the error\n";
			void tls_error();
		}

		SSL_free(serverOptions->getdataSSL(sid));
	}
	ShutdownConnection(serverOptions->getDataSock(sid));
	serverOptions->setDataSock(sid,NULL);
	serverOptions->setdataSSL(sid,NULL);

	tcpman->sendline(sd,226, "Transfer Complete",tls_cnx);
	return;

}

void parser::CDUP(SOCKET sd,ftp_options *serverOptions, int sid)
{
	SSL *tls_cnx = serverOptions->getSSL(sid);

	if(strlen(serverOptions->getsubdirs(sid)))
	{
		//std::cout<<"we have subdirs\n";
		serverOptions->delsubdir(sid);
		tcpman->sendline(sd,250,"CWD command successful",tls_cnx);
		return;
	}


	if(serverOptions->getcurrentdir(sid)->getUp() != NULL)
	{
		if(authGroupListAtLevel(serverOptions->getcurrentdir(sid), serverOptions->getusers(sid)->getgroup()))
		{
			serverOptions->setcurrentdir(sid,serverOptions->getcurrentdir(sid)->getUp());
			tcpman->sendline(sd,250,"CWD command successful",tls_cnx);
			return;
		}
		else
		{
			tcpman->sendline(sd,550,"Access denied",tls_cnx);
			return;
		}
	}

	tcpman->sendline(sd,550,"Directory not found",tls_cnx);

}



int parser::fullPath(char *path, char **path_cpy, char **tpath, ftp_options *serverOptions, int sid)
{

	//this is fucked up and needs to be fixed

	char *next_token;
	char *token;
	char *next_token2;
	//CWD  <SP> <pathname> <CRLF>
	*tpath = (char*)mymalloc(strlen(path)+1);
	strcpy_s(*tpath,strlen(path)+1,path);

	//std::cout<<"path is "<<path<<" tpath is "<<*tpath<<"\n";

	folder *tmp;
	folder *current;
	char *currentpath;
	current = serverOptions->getcurrentdir(sid);
	char *a = serverOptions->getsubdirs(sid);

	currentpath = (char*)mymalloc((strlen(a)+1)*sizeof(char));
	strcpy_s(currentpath,strlen(a)+1,a);


	//cout<<"token is:"<<path<<"\n";
	//if total path is given
	if((path[0] == '/') && (countslash(strlen(path),path) != 1))
	{
		serverOptions->setsubdirs(sid,"");
		tmp = serverOptions->getdtree(ROOTGROUP);

		token = strtok_s(path, "/", &next_token);
		token = fixCR(token);
		token = strtok_s(NULL, "/", &next_token);
		token = fixCR(token);

		*path_cpy = strtok_s(*tpath, "/", &next_token2);
		*path_cpy = fixCR(*path_cpy);

		//cout<<"token is:"<<token<<"\n";
	}
	else
	{
		if(countslash(strlen(path),path) == 1)
		{
			*path_cpy = strtok_s(*tpath,"/",&next_token2);
		}
		else
		{
			*path_cpy = strtok_s(*tpath,"",&next_token2);
		}
		*path_cpy = fixCR(*path_cpy);
		return 0;
	}


	//		cout<<"argument is "<<token<<" a length "<<strlen(token)<<"\n";
	//if(!strcmp(token,serverOptions->gettotalpath(sid)))
	if(false)
	{
		//in that dir already
		//tcpman->sendline(sd,250,"CWD command successful",tls_cnx);
		return 0;
	}
	else
	{

		do
		{

			int NL = 0;
			if(tmp != NULL)
			{
				tmp = tmp->getDown();
			}
			while(tmp != NULL)
			{
				//std::cout<<"tmp = "<<tmp<<"\n";
				if(!strcmp(*path_cpy,tmp->getName()))
				{


					serverOptions->setcurrentdir(sid,tmp);
					NL = 1;
					break;


				}
				tmp = tmp->getNext();

			};
			//cout<<"NL is "<<NL<<"\n";
			if(!NL)
			{
				////std::cout<<"the folder is "<<tmp->getName()<<"\n";
				if(authGroupList(serverOptions->getcurrentdir(sid),serverOptions->getusers(sid)->getgroup()))
				{
					if(checkdir(serverOptions->gettotalrealpath(sid),*path_cpy,serverOptions->getcurrentdir(sid),serverOptions->getusers(sid)))
					{
						serverOptions->addsubdir(sid,*path_cpy);
						//tcpman->sendline(sd,250,"CWD command successful",tls_cnx);
						//return 0;

						NL = 1;
					}
				}
				else
				{
					//tcpman->sendline(sd,550,"Access denied",tls_cnx);
					serverOptions->setcurrentdir(sid,current);
					serverOptions->setsubdirs(sid,currentpath);
					free (currentpath);
					free (*tpath);
					return ACCESSDENIED;
				}
			}
			if(!NL)
			{
				//not found
				//tcpman->sendline(sd,550,"Directory not found",tls_cnx);
				serverOptions->setcurrentdir(sid,current);
				serverOptions->setsubdirs(sid,currentpath);
				free (currentpath);
				free (*tpath);
				return DIRECTORYNOTFOUND;
			}



			token = strtok_s(NULL, "/", &next_token);
			token = fixCR(token);

			*path_cpy = strtok_s(NULL, "/", &next_token2);

			*path_cpy = fixCR(*path_cpy);

			//std::cout<<"path_cpy is "<<*path_cpy<<"\n";
			////std::cout<<"token is "<<token<<"\n";
		}
		while(token != NULL);

		//in that dir already
		//			tcpman->sendline(sd,250,"CWD command successful",tls_cnx);


	}
	//CWD command successful. "/home/Misc" is current directory.


	return 0;



}