/***********************************************
** site.cpp            horseshoeFTPd          **
** Nick Byra                                  **
** 21/03/06                                   **
**                                            **
***********************************************/

#include "user.h"
#include "xml_manager.h"
#include "site.h"
#include "consts.h"
#include "helpfunctions.h"
#include "auth.h"
#include "logging.h"
#include "crypt.h"
#include "ls.h"
#include "mem_manage.h"


int siteParser(char *line, int sid, ftp_options *serverOptions)
{
	char *next_token;
	char *token;

	token = strtok_s(line," ",&next_token);

	if(!strcmp(token,"setbwlimit"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setBWlimit(_atoi64(token));
		return SETBWLIMIT + SUCCESSFUL;
	}
	else if(!strcmp(token,"setbwlimittype"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setBWlimitType(atoi(token));
		return SETBWLIMITTYPE + SUCCESSFUL;
	}
	else if(!strcmp(token,"setip"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token); 
	}
	else if(!strcmp(token,"setname"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkAlphaNumeric(token,50))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setName(token);
		return SETNAME + SUCCESSFUL;
	}
	else if(!strcmp(token,"sslon"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		serverOptions->setusessl(true);
		return SSLON + SUCCESSFUL;
	}
	else if(!strcmp(token,"ssloff"))
	{
	if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		serverOptions->setusessl(false);
		return SSLOFF + SUCCESSFUL;
	}
	else if(!strcmp(token,"setdataporth"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setDataportrangeH(atoi(token));
		return SETDATAPORTH + SUCCESSFUL;
	}
	else if(!strcmp(token,"setdataportl"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setDataportrangeL(atoi(token));
		return SETDATAPORTL + SUCCESSFUL;
	}
	else if(!strcmp(token,"setfreefilesize"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setFreefilesize(atoi(token));
		return SETFREEFILESIZE + SUCCESSFUL;
	}
	else if(!strcmp(token,"settimeout"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setIdletimeout(atoi(token));
		return SETTIMEOUT + SUCCESSFUL;
	}
	else if(!strcmp(token,"setconnectionsperip"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setMaxconnectionsperip(atoi(token));
		return SETCONNECTIONSPERIP + SUCCESSFUL;
	}
	else if(!strcmp(token,"setmaxusers"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setMaxusers(atoi(token));
		return SETMAXUSERS + SUCCESSFUL;
	}
	else if(!strcmp(token,"setpacketsize"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setPacketsize(atoi(token));
		return SETPACKETSIZE + SUCCESSFUL;
	}
	else if(!strcmp(token,"setport"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		serverOptions->setPort(atoi(token));
		return SETPORT + SUCCESSFUL;
	}	
	else if(!strcmp(token,"adddir"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL,"",&next_token);
		int tlen = strlen(token)+1;
		char *tline = (char*)mymalloc(tlen);
		char *tdir = (char*)mymalloc(tlen);
		strcpy_s(tline,tlen,token);
		char *ttok = strtok_s(tline," ",&next_token);

		strcpy_s(tdir,tlen,"");

		while(ttok !=NULL)
		{
			token = ttok;

			ttok = strtok_s(NULL," ",&next_token);
			if(ttok != NULL)
			{
				if(strlen(tdir) > 0)
				{
					strcat_s(tdir,tlen," ");
				}
				strcat_s(tdir,tlen,token);
			}
		};

		//std::cout<<tdir<<"\n";

		if(!checkRealdir(tdir,""))
		{
			free (tdir);
			return ADDDIR + DIRNOTFOUND;
		}
		folder *fldr = new folder();
		char *tmp = (char*)mymalloc(strlen(tdir)+2);
		sprintf_s(tmp,strlen(tdir)+2,"%s\\",tdir);
		fldr->setLocation(tmp);
		free (tmp);
		free (tdir);

		if(!checkAlphaNumeric(token,0))
		{
			free(tline);
			delete fldr;
			return SYNTAXINVALID;
		}

		fldr->setName(token);
		group *grp = new group(ROOTGROUP,READACCESS,NULL);
		fldr->setGfirst(grp);

		if(serverOptions->getcurrentdir(sid)->getDown() == NULL)
		{
			serverOptions->getcurrentdir(sid)->setDown(fldr);
			fldr->setUp(serverOptions->getcurrentdir(sid));
		}
		else
		{
			addDir(serverOptions->getcurrentdir(sid)->getDown(), fldr);
		}

		forceDirSave(serverOptions);
		free(tline);
		return ADDDIR + SUCCESSFUL;

	}
	else if(!strcmp(token,"adddirgroup"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL," ",&next_token);

		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		group *grp = new group();
		grp->setName(atoi(token));
		token = strtok_s(NULL," ",&next_token);

		if(!checkNumeric(token))
		{
			return SYNTAXINVALID;
		}
		grp->setPremissions(atoi(token));

		if(addGroupToDir(serverOptions->getcurrentdir(sid), grp))
		{
			return ADDDIRGROUP + SUCCESSFUL;
		}
		else
		{
			delete grp;
			return ADDDIRGROUP + ALREADYEXISTS;
		}

	}
	else if(!strcmp(token,"deldir"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL," ",&next_token);

		if(!checkAlphaNumeric(line,0))
		{
			return SYNTAXINVALID;
		}

		if(!delDir(serverOptions->getcurrentdir(sid)->getDown(), token))
		{
			return DELDIR + DIRNOTFOUND;
		}

		return DELDIR + SUCCESSFUL;


	}
	else if(!strcmp(token,"editusergroup"))
	{
		//code me
	}
	else if(!strcmp(token,"editdir"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}

		token = strtok_s(NULL," ",&next_token);

		if(!checkRealdir(token,""))
		{
			return EDITDIR + DIRNOTFOUND;
		}

		char *tmp = (char*)mymalloc(strlen(token)+2);
		sprintf_s(tmp,strlen(token)+2,"%s\\",token);

		token = strtok_s(NULL," ",&next_token);

		if(!checkAlphaNumeric(token,0))
		{
			free (tmp);
			return SYNTAXINVALID;
		}

		serverOptions->getcurrentdir(sid)->setLocation(tmp);
		free (tmp);
		serverOptions->getcurrentdir(sid)->setName(token);

		return EDITDIR + SUCCESSFUL;



	}
	else if(!strcmp(token,"adduser"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}


		char *name;
		char *password;
		int group;
		char tmp[41];



		user *myuser = new user();
		token = strtok_s(NULL," ",&next_token);
		if(!checkAlphaNumeric(token,29))
		{
			delete myuser;
			return SYNTAXINVALID;
		}
		//strcpy_s(tmp,41,token);
		myuser->setname(token);
		token = strtok_s(NULL," ",&next_token);
		if(!checkAlphaNumeric(token,19))
		{
			delete myuser;
			return SYNTAXINVALID;
		}
		getsha1hash(token,tmp,strlen(token));
		myuser->setpassword(tmp);
		token = strtok_s(NULL," ",&next_token);
		if(!checkAlphaNumeric(token,29))
		{
			delete myuser;
			return SYNTAXINVALID;
		}
		//strcpy_s(tmp,41,token);
		if(!authIsGroup(token))
		{
			delete myuser;
			return ADDUSER + INVALIDGROUP;
		}

		myuser->setgroupname(token);

		myuser->loadDefault();
		myuser->addIPrules(new IPrules("*.*.*.*",true));

		xml_manager *myxml = new xml_manager("user.xml");
		myxml->newUser(myuser);	
		delete myxml;

		delete myuser;

		return ADDUSER + SUCCESSFUL;

	}
	else if(!strcmp(token,"addgroup"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}

		char *name;
		int group;
		char tmp[30];

		usergroup *myusergroup = new usergroup();
		token = strtok_s(NULL," ",&next_token);
		if(!checkAlphaNumeric(token,29))
		{
			delete myusergroup;
			return SYNTAXINVALID;
		}
		strcpy_s(tmp,30,token);
		myusergroup->setname(tmp);

		token = strtok_s(NULL," ",&next_token);
		if(!checkNumeric(token))
		{
			delete myusergroup;
			return SYNTAXINVALID;
		}
		myusergroup->setgroup(atoi(token));

		myusergroup->loadDefault();


		xml_manager *myxml = new xml_manager("group.xml");
		myxml->newUsergroup(myusergroup);	
		delete myxml;

		delete myusergroup;

		return ADDGROUP + SUCCESSFUL;

	}
	else if(!strcmp(token,"delgroup"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL," ",&next_token);

		if(authGroupUsed(token))
		{
			return DELGROUP + GROUPINUSE;
		}


		xml_manager *myxml = new xml_manager("group.xml");
		int rval = myxml->delUser(token);
		delete myxml;

		return DELGROUP + rval;

	}
	else if(!strcmp(token,"deluser"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}
		token = strtok_s(NULL," ",&next_token);

		user *tuser = serverOptions->getUserByName(token);

		if(tuser != NULL)
		{
			return DELUSER + USERONLINE;
		}

		xml_manager *myxml = new xml_manager("user.xml");
		int rval = myxml->delUser(token);
		delete myxml;

		return DELUSER + rval;

	}
	else if(!strcmp(token,"editgroup"))
	{
		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}

		token = strtok_s(NULL," ",&next_token);

		xml_manager *myxml = new xml_manager("group.xml");
		usergroup *mygroup = myxml->loadGroup(token);


		if(mygroup==NULL)
		{
			delete myxml;
			return USERNOTFOUND;
		}

		token = strtok_s(NULL," ",&next_token);

		if(!strcmp(token,"allowfxp"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setfxp(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGEALLOWFXP,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else if(!strcmp(token,"bwlimit"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setBWlimit(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGEBWLIMIT,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else if(!strcmp(token,"bwlimittype"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setBWlimitType(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGEBWLIMITTYPE,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else if(!strcmp(token,"iponfxp"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setIPfxp(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGEIPONFXP,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else if(!strcmp(token,"freefilesize"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setfreeFileSize(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGEFREEFILESIZE,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else if(!strcmp(token,"permissions"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setgroup(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGEPREMISSIONS,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));		
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else if(!strcmp(token,"maxdownloadspeed"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setmaxDownSpeed(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGEMAXDOWNLOADSPEED,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));		
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else if(!strcmp(token,"maxlogins"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setmaxLogins(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGEMAXLOGINS,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else if(!strcmp(token,"maxuploadspeed"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setmaxUpSpeed(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGEMAXUPLOADSPEED,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));		
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else if(!strcmp(token,"ratiod"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setratio_d(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGERATIOD,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));	
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else if(!strcmp(token,"ration"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				delete myxml;
				delete mygroup;
				return SYNTAXINVALID;
			}
			mygroup->setratio_n(atoi(token));
			myxml->saveUsergroup(mygroup);
			delete myxml;
			Log(SETTINGCHANGE,USERGROUP,serverOptions->getusers(sid)->getname(),CHANGERATION,mygroup->getname(),COMPLETE,token,serverOptions->getremoteIP(sid));
			delete mygroup;
			return EDITGROUP + SUCCESSFUL;
		}
		else
		{
			delete myxml;
			delete mygroup;
			return SYNTAXINVALID;
		}


	}
	else if(!strcmp(token,"edituser"))
	{

		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}

		token = strtok_s(NULL," ",&next_token);

		user *myuser = serverOptions->getUserByName(token);
		xml_manager *myxml;

		if(myuser == NULL)
		{
			myxml = new xml_manager("user.xml");
			myuser = myxml->loadUser(token);
			delete myxml;
		}


		if(myuser==NULL)
		{

			return USERNOTFOUND;
		}

		token = strtok_s(NULL," ",&next_token);

		if(!strcmp(token,"active"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{

				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			myuser->setactive(atoi(token));
			if(myuser != NULL)
			{
				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}

			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;
		}
		else if(!strcmp(token,"addip"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkIPsyntax(token))
			{

				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			IPrules *tip = new IPrules(token,true);
			if(!myuser->addIPrules(tip))
			{

				delete tip;
				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return EDITUSER + ADDIPFAILED;
			}
			if(myuser != NULL)
			{
				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}

			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;

		}
		else if(!strcmp(token,"delip"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkIPsyntax(token))
			{

				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}

			if(!myuser->delIPrules(token))
			{

				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return EDITUSER + DELIPFAILED;
			}
			if(myuser != NULL)
			{
				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;

		}
		else if(!strcmp(token,"allowfxp"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}

			if(myuser != NULL)
			{
				myuser->setfxp(atoi(token));
				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;

		}
		else if(!strcmp(token,"ratiooff"))
		{
			if(myuser != NULL)
			{
				myuser->setcredits(-1);
				myuser->setratio_d(-1);
				myuser->setratio_n(-1);

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}

			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;

		}
		else if(!strcmp(token,"bwlimit"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{

				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setBWlimit(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;

		}
		else if(!strcmp(token,"bwlimittype"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{

				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setBWlimitType(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;

		}
		else if(!strcmp(token,"iponfxp"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{

				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setIPfxp(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;

		}
		else if(!strcmp(token,"credits"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{

				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setcredits(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;

		}
		else if(!strcmp(token,"freefilesize"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setfreeFileSize(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;

		}
		else if(!strcmp(token,"groupname"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkAlphaNumeric(token,29))
			{
				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setgroupname(token);

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;
		}
		else if(!strcmp(token,"permissions"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{

				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setgroup(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;
		}
		else if(!strcmp(token,"maxuploadspeed"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setmaxUpSpeed(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;
		}
		else if(!strcmp(token,"maxdownloadspeed"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{

				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setmaxDownSpeed(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;
		}
		else if(!strcmp(token,"locktogroup"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}

			if(myuser != NULL)
			{
				myuser->setlocktogroup(atoi(token));
				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;
		}
		else if(!strcmp(token,"maxlogins"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setmaxLogins(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;
		}
		else if(!strcmp(token,"password"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkAlphaNumeric(token,19))
			{
				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			char tmp[41];
			getsha1hash(token,tmp,strlen(token));
			myuser->setpassword(tmp);
			myxml->saveUser(myuser);
			delete myxml;
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;
		}
		else if(!strcmp(token,"ratiod"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setratio_d(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;
		}
		else if(!strcmp(token,"ration"))
		{
			token = strtok_s(NULL," ",&next_token);
			if(!checkNumeric(token))
			{
				if(serverOptions->getUserByName(myuser->getname()) == NULL)
				{
					delete myuser;
				}
				return SYNTAXINVALID;
			}
			if(myuser != NULL)
			{
				myuser->setratio_n(atoi(token));

				myuser->forcesave();
			}
			else
			{
				return EDITUSER + UNKNOWNERROR;
			}
			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return EDITUSER + SUCCESSFUL;
		}
		else
		{

			if(serverOptions->getUserByName(myuser->getname()) == NULL)
			{
				delete myuser;
			}
			return SYNTAXINVALID;
		}


	}
	else if(!strcmp(token,"kill"))
	{

		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}

		token = strtok_s(NULL," ",&next_token);

		if(!checkAlphaNumeric(token,29))
		{
			return SYNTAXINVALID;
		}

		bool FLAG = false;
		for(int i=0;i<serverOptions->getMaxusers();i++)
		{
			if(serverOptions->getusers(i) != NULL)
			{
				if(!strcmp(serverOptions->getusers(i)->getname(),token))
				{
					serverOptions->setDIEBITCH(i,true);
					FLAG = true;
				}
			}
		}

		if(FLAG)
		{
			return KILLUSER + SUCCESSFUL;
		}
		else
		{
			return KILLUSER + USERNOTFOUND;
		}

	}
	else if(!strcmp(token,"shutdown"))
	{

		if(serverOptions->getusers(sid)->getgroup() != ROOTGROUP)
		{
			return ACCESSDENIED;
		}

		serverOptions->setSHUTDOWN(true);

		return SERVERSHUTDOWN + SUCCESSFUL;


	}
	else
	{
		return SYNTAXINVALID;
	}

	return SYNTAXINVALID;

}

