/***********************************************
** tcp_manager.cpp     horseshoeFTPd          **
** Nick Byra                                  **
** 21/03/06                                   **
**                                            **
***********************************************/

#include "tcp_manager.h"
#include "parser.h"
#include "auth.h"
#include "user.h"
#include <io.h>
#include "ws-util.h"
#include <iostream>
#include <assert.h>
#include <string>
#include "helpfunctions.h"
#include "consts.h"
#include "logging.h"
#include "mem_manage.h"

using namespace std;



typedef struct _MyData {
	tcp_manager *tcpman;
	u_long remoteip;
	ftp_options *serverOptions;
	SOCKET sd;
} MYDATA, *PMYDATA;



tcp_manager::tcp_manager(ftp_options *a)
{
	serverOptions = a;

	//this is the abort sequence we need to check for
	ABORT[0]= -1;
	ABORT[1]= -14;
	ABORT[2]= -1;
	ABORT[3]= 65;
	ABORT[4]= 66;
	ABORT[5]= 79;
	ABORT[6]= 82;
	ABORT[7]= 13;
	ABORT[8]= 10;
	ABORT[9]= '\0';


}

tcp_manager::~tcp_manager()
{
}



int tcp_manager::DoWinsock(const char* pcAddress, int nPort)
{
	//cout << "Establishing the listener..." << endl;
	//create a socket to listen on
	SOCKET ListeningSocket = SetUpListener(pcAddress, htons(nPort));
	//check to see if socket created successfully
	if (ListeningSocket == INVALID_SOCKET) 
	{
		//cout << endl << WSAGetLastErrorMessage("establish listener") << endl;
		return 3;
	}

	//loop waiting from connections
	//cout << "Waiting for connections..." << flush;
	while (1) {
		AcceptConnections(ListeningSocket);
		//cout << "Acceptor restarting..." << endl;
	}


	return 0; 

}


SOCKET tcp_manager::EstablishConnection(u_long nRemoteAddr, u_short nPort)
{

	// Create a stream socket
	SOCKET sock = socket(AF_INET, SOCK_STREAM, 0);

	if (sock != INVALID_SOCKET) 
	{
		sockaddr_in sinRemote;
		sinRemote.sin_family = AF_INET;
		sinRemote.sin_addr.s_addr = nRemoteAddr;
		sinRemote.sin_port = htons(nPort);

		if (connect(sock, (sockaddr*)&sinRemote, sizeof(sockaddr_in)) == SOCKET_ERROR) 
		{
			cerr << WSAGetLastErrorMessage("connect() failed") <<
				endl;
			return SOCKET_ERROR;
		}



	}
	else
	{
		return SOCKET_ERROR;
	}

	return sock;
}

SOCKET tcp_manager::SetUpConnect(const char* pcAddress, u_short nPort, int sid)
{


	SSL_CTX *tls_ctx = serverOptions->getctx();
	SSL *tls_cnx = NULL;
	//cout<<pcAddress<<"\n";
	//strcpy(pcAddress,pc);

	//convert string of ip to long
	u_long nInterfaceAddr = inet_addr(pcAddress);
	//check from invalid ip
	if (nInterfaceAddr != INADDR_NONE) 
	{
		//create socket
		SOCKET sd = EstablishConnection(nInterfaceAddr,nPort);

		if(serverOptions->getusessl())
		{
			tls_cnx = setupcnx(setupbio(sd), tls_ctx,CONNECT);
			serverOptions->setdataSSL(sid,tls_cnx);

		}
		return sd;
	}

	return SOCKET_ERROR;
}







SOCKET tcp_manager::SetUpListener(const char* pcAddress, int nPort)
{
	//convert string of ip to long
	u_long nInterfaceAddr = inet_addr(pcAddress);
	//check from invalid ip
	if (nInterfaceAddr != INADDR_NONE) {
		//create socket
		SOCKET sd = socket(AF_INET, SOCK_STREAM, 0);
		//check if socket is valid
		if (sd != INVALID_SOCKET) {
			//setup other socet data
			sockaddr_in sinInterface;
			sinInterface.sin_family = AF_INET;
			sinInterface.sin_addr.s_addr = nInterfaceAddr;
			sinInterface.sin_port = nPort;
			//bind socket
			if (bind(sd, (sockaddr*)&sinInterface, 
				sizeof(sockaddr_in)) != SOCKET_ERROR) {
					//setup listener
					listen(sd, SOMAXCONN);
					//return socket
					return sd;
			}
			else {
				cerr << WSAGetLastErrorMessage("bind() failed") <<
					endl;
			}
		}
	}

	return INVALID_SOCKET;
}





SOCKET tcp_manager::SetUpPasvListener(const char* pcAddress, int *nPort)
{
	//convert string of ip to long
	u_long nInterfaceAddr = inet_addr(pcAddress);
	//check from invalid ip
	if (nInterfaceAddr != INADDR_NONE) {
		//create socket
		SOCKET sd = socket(AF_INET, SOCK_STREAM, 0);
		//check if socket is valid
		if (sd != INVALID_SOCKET) {
			//setup other socet data
			sockaddr_in sinInterface;
			sinInterface.sin_family = AF_INET;
			sinInterface.sin_addr.s_addr = nInterfaceAddr;

			//seeding random
			srand( (unsigned)time( NULL ) );

			int firsttry = serverOptions->getDataportrangeL() + rand() % (serverOptions->getDataportrangeH() - serverOptions->getDataportrangeL() + 1);
			*nPort = firsttry;

			while(1) {
				sinInterface.sin_port = htons(*nPort);

				//try to bind socket
				if (bind(sd, (sockaddr*)&sinInterface, sizeof(sockaddr_in)) != SOCKET_ERROR) 
				{
					break;
				}
				*nPort--;
				if (*nPort < serverOptions->getDataportrangeL()) 
				{
					*nPort = serverOptions->getDataportrangeH();
				}
				if (*nPort == firsttry)
				{
					_close(sd);

					return ALLPORTSBUSY;
				}


			}
			//setup listener
			listen(sd, 1);
			//return socket
			return sd;

		}
	}

	return INVALID_SOCKET;
}





DWORD WINAPI ConnectionHandler(LPVOID lpParam ) 
{
	int nRetval = 0;
	int sid = NOSID;
	bool CONNECTED = false;
	//SOCKET sd = (SOCKET)pss_;
	char *tmp;
	tmp = (char *)mymalloc(255);
	PMYDATA pData;
	pData = (PMYDATA)lpParam;
	//create user object
	user *sUser = NULL;
	SSL_CTX *tls_ctx = NULL;
	SSL *tls_cnx = NULL;


	HANDLE hStdout;


	hStdout = GetStdHandle(STD_OUTPUT_HANDLE);


	if(pData->serverOptions->getusessl())
	{
		tls_init_library();

		tls_ctx = setup_server_ctx();
		pData->serverOptions->setctx(tls_ctx);
		tls_cnx = setupcnx(setupbio(pData->sd), tls_ctx,ACCEPT);
	}




	if(authServerFull(pData->serverOptions) == AUTHORIZED)
	{
		CONNECTED = true;
		sid = pData->serverOptions->incCurrentUsers();
		pData->serverOptions->setSSL(sid,tls_cnx);
		pData->serverOptions->setremoteIP(sid,pData->remoteip);
	}
	else
	{
		Log(LOGIN,"",SERVERFULL,pData->remoteip);
		CONNECTED = false;
		pData->tcpman->sendline(pData->sd, 421, "Server Full",tls_cnx);
	}






	int sendn = 0;


	if(CONNECTED)
	{

		//its 4:20 am and i'm just screwing around, not thats not a pot joke, it really just happened to be 4:20
		//should probobly comment these out before the release
		sendn = pData->tcpman->sendline(pData->sd, -1, "220-99 Kriegsminister",tls_cnx);
		sendn = pData->tcpman->sendline(pData->sd, -1, "220-Streichholz und Benzinkanister",tls_cnx);
		sendn = pData->tcpman->sendline(pData->sd, -1, "220-Hielten sich fuer schlaue Leute",tls_cnx);
		sendn = pData->tcpman->sendline(pData->sd, -1, "220-Witterten schon fette Beute",tls_cnx);
		sendn = pData->tcpman->sendline(pData->sd, -1, "220-Riefen: Krieg und wollten Macht ",tls_cnx);
		sendn = pData->tcpman->sendline(pData->sd, -1, "220-Mann, wer haette das gedacht",tls_cnx);
		sendn = pData->tcpman->sendline(pData->sd, -1, "220-Dass es einmal soweit kommt ",tls_cnx);
		sendn = pData->tcpman->sendline(pData->sd, -1, "220-Wegen 99 Luftballons",tls_cnx);

		//send connected to server message
		sendn = pData->tcpman->sendline(pData->sd, 220, "Connected to server - horseshoeFTPd",tls_cnx);




	}

	parser *myParser = new parser(pData->tcpman);
	//= pData->serverOptions->getparser();

	while(CONNECTED)
	{
		//cout<<"loop\n";
		//clear the data buffer to aviod issues 
		clearbuff(tmp, 255);

		SOCKET psd;
		if(sUser != NULL)
		{
			psd = pData->serverOptions->getDataSock(sid);
		}
		else
		{
			psd = NULL;
		}

		//calling readline and waither for data on one of the connections
		if(int nbytes = pData->tcpman->readline(pData->sd,tmp, 255,pData->serverOptions->getIdletimeout(),sid) > 0)
		{
			int code;
			if(nbytes == FORCINGDISCONNECT)
			{
				CONNECTED = false;
			}
			else
			{
				//sending received data to the parser
				code = myParser->parse(pData->sd, tmp, &sUser,pData->serverOptions, sid);
			}

			if(sUser == NULL)
			{
				//cout<<"i still == null\n";
			}
			//shutting down connection if code is QUIT, will probobly move this to parser
			if(code == QUIT)
			{
				//cout << "Shutting connection down..." << flush;

				CONNECTED = false;
			}

		}
		else if(nbytes == -2)
		{
			//closing the connection in the result of a timeout
			//cout << "Shutting connection down for timeout..." << flush;

			CONNECTED = false;

		}
		else
		{
			//cout<<"socket error\n";
			CONNECTED = false;
		}

	};



	if(sid != NOSID)
	{
		//cleaning up the ssl objects
		if(pData->serverOptions->getusessl())
		{
			SSL_shutdown(pData->serverOptions->getSSL(sid));
			SSL_free(pData->serverOptions->getSSL(sid));

			//incase there is something there

			if(pData->serverOptions->getdataSSL(sid) != NULL)
			{
				SSL_shutdown(pData->serverOptions->getdataSSL(sid));
				SSL_free(pData->serverOptions->getdataSSL(sid));
			}
		}


		if (ShutdownConnection(pData->sd))
		{
			cout << "Connection is down." << endl;
		}

		if(sUser != NULL)
		{
			Log(LOGOFF,sUser->getname(),COMPLETE,pData->remoteip);
			pData->serverOptions->remUsers(sUser,sid);
		}
		else
		{
			Log(LOGOFF,"",COMPLETE,pData->remoteip);
		}

		pData->serverOptions->decCurrentUsers(sid);

	}


	delete myParser;

	HeapFree(GetProcessHeap(), 0, pData);

	return nRetval;
}





SOCKET tcp_manager::AcceptPasvConnections(SOCKET ListeningSocket,int sid)
{
	//create sockaddr for the remote commection
	sockaddr_in sinRemote;
	int nAddrSize = sizeof(sinRemote);

	int tmer = 0;
	while (tmer < 60)
	{
		tmer++;

		fd_set fd;
		FD_ZERO(&fd);
		FD_SET(ListeningSocket,&fd);
		TIMEVAL tv = {1, 0};

		if(select(0, &fd, NULL, NULL, &tv) != 0)
		{
			//there is data on one of the sockets to be read
			if (FD_ISSET(ListeningSocket, &fd))
			{

				SOCKET sd = accept(ListeningSocket, (sockaddr*)&sinRemote,
					&nAddrSize);
				//std::cout<<"in accept loop\n";
				if (sd != INVALID_SOCKET) {
					cout << "Accepted connection from " <<
						inet_ntoa(sinRemote.sin_addr) << ":" <<
						ntohs(sinRemote.sin_port) << "." <<
						endl;
					if(closesocket(ListeningSocket)==SOCKET_ERROR)
					{
						cerr << WSAGetLastErrorMessage("closesocket() failed") << 
							endl;
					}

					serverOptions->getusers(sid)->printValues();
					if(!(serverOptions->getusers(sid)->getfxp()))
					{
						//std::cout<<"in fxp pasv\n";
						if(serverOptions->getremoteIP(sid) != sinRemote.sin_addr.S_un.S_addr)
						{
							return FXPNOTALLOWED;
						}

					}


					return sd;
				}
				else {
					//if accept bad print error
					cerr << WSAGetLastErrorMessage("accept() failed") << 
						endl;
					return NULL;
				}
			}
		}
	};

	return TIMEOUT;
}


void tcp_manager::AcceptConnections(SOCKET ListeningSocket)
{
	//create sockaddr for remote connection
	sockaddr_in sinRemote;
	int nAddrSize = sizeof(sinRemote);
	//loop till connection
	while (1) {	
		SOCKET sd = accept(ListeningSocket, (sockaddr*)&sinRemote,
			&nAddrSize);
		if (sd != INVALID_SOCKET) {
			cout << "Accepted connection from " <<
				inet_ntoa(sinRemote.sin_addr) << ":" <<
				ntohs(sinRemote.sin_port) << "." <<
				endl;
			//create thread id
			DWORD nThreadID;
			//thread off socket to ConnectionHandler
			PMYDATA pData;

			pData = (PMYDATA)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(MYDATA));
			pData->sd = sd;
			pData->tcpman = this;
			pData->remoteip = sinRemote.sin_addr.S_un.S_addr;
			pData->serverOptions = serverOptions;
			Log(CONNECTION,sinRemote.sin_addr.S_un.S_addr);
			//ConnectionHandler( (void*)sd );
			CreateThread(0, 0, ConnectionHandler, pData, 0, &nThreadID);
		}
		else {
			//print error if accept failed
			cerr << WSAGetLastErrorMessage("accept() failed") << 
				endl;
			return;
		}
	}
}


SOCKET tcp_manager::DoPasv(SOCKET sd, int sid, ftp_options *serverOptions)
{

	SSL_CTX *tls_ctx = serverOptions->getctx();
	SSL *tls_cnx = NULL;
	int nPort;
	cout << "Establishing the pasv listener..." << endl;

	//create a socket to listen on for the passive
	SOCKET ListeningSocket = SetUpPasvListener(serverOptions->getIP(), &nPort);
	if (ListeningSocket == INVALID_SOCKET) {
		//print error if create socket failed
		cout << endl << WSAGetLastErrorMessage("establish listener") << 
			endl;
		return NULL;
	}

	int a = nPort/256;
	int b = nPort%256;

	char tmp[50];
	char tip[16];
	strcpy_s(tip,16,serverOptions->getIP());

	for(int i=0;i<strlen(tip);i++)
	{
		if(tip[i]=='.')
		{
			tip[i] = ',';
		}
	}

	sprintf_s(tmp,50,"Entering Passive Mode (%s,%i,%i)",tip,a,b);


	//sending the message to the client that passive mode is on
	//this is in here not the parser because of timming issues
	sendline(sd,227,tmp,serverOptions->getSSL(sid));
	SOCKET psd = AcceptPasvConnections(ListeningSocket,sid);

	//std::cout<<"the socket returned from pasv is "<<psd<<"\n";

	if(psd == FXPNOTALLOWED)
	{
		sendline(sd,553,"FXP not allowed on this server",serverOptions->getSSL(sid));
		ShutdownConnection(psd);
		return FXPNOTALLOWED;
	}

	if(psd == TIMEOUT)
	{
		sendline(sd,553,"PASV timed out",serverOptions->getSSL(sid));
		ShutdownConnection(psd);
		return TIMEOUT;
	}

	if(serverOptions->getusessl())
	{

		tls_cnx = setupcnx(setupbio(psd), tls_ctx,ACCEPT);
		serverOptions->setdataSSL(sid,tls_cnx);
	}

	serverOptions->setDataSock(sid,psd);


	return psd;  

}









int tcp_manager::readline(SOCKET sd, char *pch, const int nSize, const int nSecs,int sid)
{
	//cout<<"in recieve \n";
	SSL *tls_cnx = serverOptions->getSSL(sid);


	assert(sd != NULL);
	fd_set fd;
	fd_set efd;



	int cnt = 0;
	int bufferat = 0;
	char *tmp;
	int tmer = 0;

	//will go though loop once aslong as nSecs is not a negative value
	while(tmer <= nSecs)
	{

		if(serverOptions->getDIEBITCH(sid))
		{
			return FORCINGDISCONNECT;
		}

		tmer++;


		//zeroing out the fd set
		FD_ZERO(&fd);
		FD_ZERO(&efd);

		FD_SET(sd,&fd);
		FD_SET(sd,&efd);


		int sval = 1;

		if(nSecs == 0)
		{
			sval = 0;
		}

		TIMEVAL tv = {sval, 0};

		if(select(0, &fd, NULL, NULL, &tv) != 0)
		{

			if (FD_ISSET(sd, &fd))
			{

				int nBytesReceived;
				int readto;
				//peaking on the socket (looking at data without removing it from the buffer
				if(bufferat)
				{
					tmp = (char*)mymalloc(bufferat+nBytesReceived);
					strcpy_s(tmp,bufferat+nBytesReceived,pch);
					clearbuff(pch, nSize);
				}

				if(bufferat)
				{
					//making sure we are not going to over write the buffer

					if( (bufferat+nBytesReceived) < nSize)
					{


						//creating a tmp buffer reading new data and appending data

						if((nBytesReceived = tls_recv(true, sd, pch, nBytesReceived, 0,tls_cnx)) == SOCKET_ERROR) 
						{	
							return SOCKET_ERROR;
						}

						readto = strfind(nSize, pch, '\n');
						//cout<<"got data\n";
						pch[nBytesReceived] = '\0';
						//cout<<"data "<<pch<<"\n";
						strcat_s(tmp,bufferat+nBytesReceived,pch);
						//cout<<"cat "<<tmp<<"\n";
						strcpy_s(pch,bufferat+nBytesReceived,tmp);
						//cout<<"cpy "<<pch<<"\n";
						free (tmp);
					}
					else
					{

						//buffer is over written cleaning it out and returning, will result in bad syntax most likely
						if((nBytesReceived = tls_recv(true, sd, pch, nBytesReceived, 0, tls_cnx)) == SOCKET_ERROR) 
						{	
							return SOCKET_ERROR;
						}

						return nBytesReceived;
					}

				}
				else
				{
					//no buffer to append so we just read
					if((nBytesReceived = tls_recv(true, sd, pch, nBytesReceived, 0,tls_cnx )) == SOCKET_ERROR) 
					{	
						//cout<<"socket error\n";
						return SOCKET_ERROR;
					}
					readto = strfind(nSize, pch, '\n');
					pch[nBytesReceived] = '\0';

				}

				//adding a \0 to make the buffer a string
				if(nBytesReceived > -1)
				{
					//making sure we don't segfault
					//might cause issued with parsing if not null terminated
					//worse case ends in bad syntax error
					if(nBytesReceived<nSize)
					{
						//pch[nBytesReceived+bufferat] = '\0';
					}
					//returning if we alread have our \n
					if(readto > -1)
					{
						return nBytesReceived + bufferat;
					}
					else
					{
						//setting bufferat to the place we are in the tmp buffer and now we go back to select()
						bufferat += nBytesReceived + 1;
					}
				}

			}



		}

	}

	return TIMEOUT;
}

//return the index of the first occurance of find in str
int tcp_manager::strfind(int bsize,char *str, char find)
{
	for(int i=0;i<bsize;i++)
	{
		if(str[i]==find)
		{
			return i;
		}

	}
	return -1;

}








int tcp_manager::sendline(SOCKET sd, int code, char *message, SSL *tls_cnx)
{

	int codesize = 3;
	if(code<=0)
	{
		codesize = 0;
	}

	int atm = codesize + 3 + strlen(message);



	char *msg = (char*)mymalloc((atm+1)*sizeof(char));


	msg[0] = '\0';

	if(code > 0)
	{
		_itoa_s(code,msg,atm+1,10);

	}

	if(code != -1)
	{

		strcat_s(msg,atm+1," ");

	}


	if(strlen(message))
	{
		strcat_s(msg,atm+1,message);
	}



	//hack
	if(code == -1)
	{
		strcpy_s(msg,atm+1,message);
	}

	strcat_s(msg,atm+1,"\r\n");
	int length = tls_send(true,sd, msg, strlen(msg), 0,tls_cnx);

	if(length < 0)
	{
		cout << endl << WSAGetLastErrorMessage("send data(sendline)") << endl;

	}

	free (msg);


	return length;

}


//this functions reads a file from the harddrive and sends it to a user
int tcp_manager::readFile(SOCKET sd, SOCKET dsd,int offset, char *fName, char *dwBuffer,int bsize, SSL *tls_cnx, int sid)
{



	//file handle
	FILE *hFile;
	//something to contain the number of bytes read
	int dwNumRead;
	//a boolean test variable, to test for success of reads
	BOOL bTest;
	char sdbuff[20];


	//open the file in a different way depending on if the file is binary or ascii
	if(serverOptions->getmode(sid) == TYPE_I)
	{
		if(fopen_s(&hFile,fName,"rb"))
		{
			//cout<<"fileopen error\n";
			return -1;
		}
	}
	else
	{
		if(fopen_s(&hFile,fName,"r"))
		{
			//cout<<"fileopen error\n";
			return -1;
		}

	}

	//do some seeking in the file if a resume needs to be done
	//if a resume is being done then offset is set the the value given by the rest command
	fseek(hFile,offset,SEEK_SET);

	//log the resume if there is one
	if(offset)
	{
		Log(TRANSFER, serverOptions->getusers(sid)->getname(), DOWNLOAD,fName, RESUMING, serverOptions->getremoteIP(sid));
	}

	//log the start of the file transfer
	Log(TRANSFER, serverOptions->getusers(sid)->getname(), DOWNLOAD,fName, STARTED, serverOptions->getremoteIP(sid));



	DWORD before;
	DWORD after = 0xffffffff;
	//start the transfer loop
	do
	{
		//do the checking for abort and diebitch
		int sd_read = this->readline(sd,sdbuff,20,0,sid);
		if(sd_read > 0)
		{
			//if we have an abort
			if(!strcmp(sdbuff,ABORT))
			{
				fclose(hFile);
				return ABORTSENT;
			}
		}

		//if we have a diebitch then we return force disconnect
		if(sd_read == FORCINGDISCONNECT)
		{
			fclose(hFile);
			return FORCINGDISCONNECT;
		}

		//if there is a socket error
		if(sd_read == SOCKET_ERROR)
		{
			fclose(hFile);
			return SOCKET_ERROR;
		}

		//read the file from disk and fill the buffer
		dwNumRead = fread(dwBuffer,sizeof(char),bsize,hFile);


		//if we read something from the file then this value will be greater then 0
		if(dwNumRead != 0)
		{
			//if there is a max download speed set then we get the tick count
			//this is the number of miliseconds since the computer has booted
			//this will reset every 49.something days
			//so basicly it will go from really big number to really small one
			//this could fuck up the math, but is not likely and shouldn't cause too much trouble so we will risk it
			if(serverOptions->getusers(sid)->getmaxDownSpeed())
			{
				before = GetTickCount();
			}
			//set up the select stuff
			fd_set fd;
			FD_ZERO(&fd);
			FD_SET(dsd,&fd);
			int snt;
			TIMEVAL tv = {60, 0};
			//call select, this is to avoid hanging here forever if the cleint dies
			if(select(0, NULL,&fd, NULL, &tv) != 0)
			{
				//if there is writeability on the data socket
				if (FD_ISSET(dsd, &fd))				
				{
					//send the fun data to the client
					snt = tls_send(true,dsd,dwBuffer,dwNumRead,0,tls_cnx);

				}
			}
			else
			{
				//we have a timout on the data socket
				//client must have died or something like that
				fclose(hFile);
				Log(TRANSFER, serverOptions->getusers(sid)->getname(), DOWNLOAD,fName, DATATIMEOUT, serverOptions->getremoteIP(sid));
				return DATATIMEOUT;
			}

			//increment the amount the user has downloaed
			if(!serverOptions->getusers(sid)->incDown(snt))
			{
				fclose(hFile);
				Log(TRANSFER, serverOptions->getusers(sid)->getname(), DOWNLOAD,fName, OUTOFCREDITS, serverOptions->getremoteIP(sid));
				return NOTENOUGHCREDITS;
			}

			//increments the users bandwidth this session
			if(!serverOptions->getusers(sid)->incBW(snt))
			{
				fclose(hFile);
				Log(TRANSFER, serverOptions->getusers(sid)->getname(), DOWNLOAD,fName, OVERUSERQUOTA, serverOptions->getremoteIP(sid));
				return OVERBANDWIDTHQUOTA;
			}

			//increment the servers bandwidth this session
			if(!serverOptions->incbwthissession(snt))
			{
				fclose(hFile);
				Log(TRANSFER, serverOptions->getusers(sid)->getname(), DOWNLOAD,fName, OVERGLOBALQUOTA, serverOptions->getremoteIP(sid));
				return OVERBANDWIDTHQUOTA;
			}

			//if there is a max download speed we do the fun math to see if we need to sleep because the transfer is going too fast
			if(serverOptions->getusers(sid)->getmaxDownSpeed())
			{
				double bwlimit = serverOptions->getusers(sid)->getmaxDownSpeed() / 1000;
				if( ((1/bwlimit)*snt) > (after - before) )
				{
					Sleep( ((1/bwlimit)*snt) - (after - before) );
				}
				after = GetTickCount();
			}

		}

	}
	while(dwNumRead != 0);

	//if there is an error reading the file
	//bad sector or something like that
	if(ferror(hFile))
	{
		Log(TRANSFER, serverOptions->getusers(sid)->getname(), DOWNLOAD,fName, ERRORREADINGFILE, serverOptions->getremoteIP(sid));
	}
	if(feof(hFile))
	{
		//end of file, transfer complete
		Log(TRANSFER, serverOptions->getusers(sid)->getname(), DOWNLOAD,fName, COMPLETE, serverOptions->getremoteIP(sid));
	}
	fclose(hFile);


	return 0;

}









//this function just sends the stuff in message to the socket sd
//its pretty simple, doesn't ask any questions
int tcp_manager::senddirlisting(SOCKET sd,char *message, int mlen,  SSL *tls_cnx)
{

	int length = tls_send(true,sd, message, mlen, 0,tls_cnx);


	if(length < 0)
	{
		cout << endl << WSAGetLastErrorMessage("send data(dirlist)") << endl;

	}

	return length;

}



ftp_options* tcp_manager::getserverOptions()
{
	return serverOptions;
}



//this functon saves a file that is being uploaded to disk
int tcp_manager::saveFile(SOCKET sd, SOCKET dsd,int offset, char *fName, char *dwBuffer, int bsize, SSL *tls_cnx, int sid)
{



	//file handle
	FILE *hFile;
	//something to contain the number of bytes read
	int dwNumRead;
	//a boolean test variable, to test for success of reads
	BOOL bTest;
	char sdbuff[20];




	//add a b to the open line if they type is I therefor binary
	char ow[4];
	strcpy_s(ow,4,"a+");
	if(serverOptions->getmode(sid) == TYPE_I)
	{
		strcat_s(ow,4,"b");
	}

	//log that the upload has started
	Log(TRANSFER, serverOptions->getusers(sid)->getname(), UPLOAD,fName, STARTED, serverOptions->getremoteIP(sid));

	//open the file and get the error code
	if(int err = fopen_s(&hFile,fName,ow))
	{
		//if the file could not be open because there is n more disk space
		if(err == ENOSPC)
		{
			Log(TRANSFER, serverOptions->getusers(sid)->getname(), UPLOAD,fName, OUTOFDISKSPACE, serverOptions->getremoteIP(sid));
			return OUTOFDISKSPACE;
		}
		else
		{
			//if it could not be open for some other reason
			Log(TRANSFER, serverOptions->getusers(sid)->getname(), UPLOAD,fName, UNKNOWNERROR, serverOptions->getremoteIP(sid));
			return UNKNOWNERROR;
		}
	}

	//create a file object to be added to the directory tree
	file *nfile = new file();
	nfile->setOwner(serverOptions->getusers(sid)->getname());
	nfile->setGroup(serverOptions->getusers(sid)->getgroup());
	nfile->setPath(serverOptions->getsubdirs(sid));
	nfile->setName(serverOptions->getincomingfile(sid));
	nfile->setDir(false);


	//find the end of the linked list and add the new file to the end of it
	file *tfile = serverOptions->getcurrentdir(sid)->getFfirst();
	if(tfile != NULL)
	{
		while(tfile->getNext() != NULL)
		{
			tfile = tfile->getNext();
		};
		tfile->setNext(nfile);
		nfile->setLast(tfile);
	}
	else
	{
		serverOptions->getcurrentdir(sid)->setFfirst(nfile);
	}

	//forceDirSave(serverOptions);

	//move to the point in the file set by offsent, if we are not resuming then this value will be zero
	fseek(hFile,offset,SEEK_SET);

	//log a resume if there is an offset
	if(offset)
	{
		Log(TRANSFER, serverOptions->getusers(sid)->getname(), UPLOAD,fName, RESUMING, serverOptions->getremoteIP(sid));
	}


	int length = 0;
	DWORD before;
	DWORD after = 0xffffffff;
	do
	{
		//make a call to readline with timeout of zero
		//this is just to check to see if an abort was sent or the shutdown bool was set
		int sd_read = this->readline(sd,sdbuff,20,0,sid);
		if(sd_read > 0)
		{
			//this is if the abort was sent
			if(!strcmp(sdbuff,ABORT))
			{
				fclose(hFile);
				Log(TRANSFER, serverOptions->getusers(sid)->getname(), UPLOAD,fName, ABORTSENT, serverOptions->getremoteIP(sid));
				return ABORTSENT;
			}
		}
		//this is if the DIEBITCH was set and we need to force a disconnect
		if(sd_read == FORCINGDISCONNECT)
		{
			fclose(hFile);
			return FORCINGDISCONNECT;
		}

		//if there is a max upload speed set then we get the tick count
		//this is the number of miliseconds since the computer has booted
		//this will reset every 49.something days
		//so basicly it will go from really big number to really small one
		//this could fuck up the math, but is not likely and shouldn't cause too much trouble so we will risk it
		if(serverOptions->getusers(sid)->getmaxUpSpeed())
		{
			before = GetTickCount();
		}

		//set up the good old select stuff
		fd_set fd;
		FD_ZERO(&fd);
		FD_SET(dsd,&fd);
		int snt;
		TIMEVAL tv = {60, 0};
		//we do select here to make sure that we do not block forever waiting for data
		//it could be a problem if the client dies

		if(select(0, &fd, NULL, NULL, &tv) != 0)
		{

			//there is data on one of the sockets to be read
			if (FD_ISSET(dsd, &fd))				
			{

				//we read in the fun data from the client
				length = tls_recv(true,dsd,dwBuffer,bsize,0,tls_cnx);

			}
		}
		else
		{
			//if we get here it means there was a timeout waiting for data so we return with the datatimeout error
			fclose(hFile);
			Log(TRANSFER, serverOptions->getusers(sid)->getname(), UPLOAD,fName, DATATIMEOUT, serverOptions->getremoteIP(sid));
			return DATATIMEOUT;
		}			
		//increase the amount uploaded for this user
		serverOptions->getusers(sid)->incUp(length);
		//increase the bandwidth this session for the server
		//if they are over the quota, return the error
		if(!serverOptions->incbwthissession(length))
		{
			fclose(hFile);
			Log(TRANSFER, serverOptions->getusers(sid)->getname(), UPLOAD,fName, OVERGLOBALQUOTA, serverOptions->getremoteIP(sid));
			return OVERBANDWIDTHQUOTA;
		}

		//increase the bandwidth this session for the user
		//return over quota if they are
		if(!serverOptions->getusers(sid)->incBW(length))
		{
			fclose(hFile);
			Log(TRANSFER, serverOptions->getusers(sid)->getname(), UPLOAD,fName, OVERUSERQUOTA, serverOptions->getremoteIP(sid));
			return OVERBANDWIDTHQUOTA;
		}

		//if max upload speed is set
		if(serverOptions->getusers(sid)->getmaxUpSpeed())
		{
			//do some math to see if we are going too fast
			double bwlimit = serverOptions->getusers(sid)->getmaxUpSpeed() / 1000;
			if( ((1/bwlimit)*length) > (after - before) )
			{
				Sleep( ((1/bwlimit)*length) - (after - before) );
			}
			after = GetTickCount();
		}

		//write the actual data to disk
		int err = fwrite(dwBuffer,sizeof(char),length,hFile);

		//check to see if there was an error writting the data
		if(err < length)
		{
			fclose(hFile);
			Log(TRANSFER, serverOptions->getusers(sid)->getname(), UPLOAD,fName, WRITEERROR, serverOptions->getremoteIP(sid));
			return UNKNOWNERROR;

		}


	}
	while(length != 0);
	//the transfer is complete, log it and return
	Log(TRANSFER, serverOptions->getusers(sid)->getname(), UPLOAD,fName, COMPLETE, serverOptions->getremoteIP(sid));
	fclose(hFile);


	return 0;

}
