/***********************************************
** xml_manager.cpp     horseshoeFTPd          **
** Nick Byra                                  **
** 21/03/06                                   **
**                                            **
***********************************************/

#include "xml_manager.h"
#include <iostream>
#include "consts.h"
#include "ls.h"
#include "helpfunctions.h"

XERCES_CPP_NAMESPACE_USE


xml_manager::xml_manager(char *xmlfile)
{
	//copy the name of the xml file to the local variable filename
	strcpy_s(filename,30,xmlfile);
	//fire up the xml utils


	//create a dom parser
	parser = new XercesDOMParser();
	parser->setValidationScheme(XercesDOMParser::Val_Always);    // optional.
	parser->setDoNamespaces(true);    // optional

	//create an error handler for the parser
	errHandler = (ErrorHandler*) new HandlerBase();
	parser->setErrorHandler(errHandler);

	//load the xml file into the parser
	try {
		parser->parse(xmlfile);
	}
	catch (const XMLException& toCatch) {
		char* message = XMLString::transcode(toCatch.getMessage());
		//std::cout << "Exception message is: \n"<< message << "\n";
		XMLString::release(&message);
		// return NULL;
	}
	catch (const DOMException& toCatch) {
		char* message = XMLString::transcode(toCatch.msg);
		//std::cout << "Exception message is: \n"<< message << "\n";
		XMLString::release(&message);
		//return NULL;
	}
	catch (...) {
		//std::cout << "Unexpected Exception \n" ;
		//return NULL;
	}


	doc = parser->getDocument();

}



xml_manager::xml_manager()
{
	//copy the name of the xml file to the local variable filename



	errHandler = NULL;
	parser = NULL;


}








xml_manager::~xml_manager()
{

	if(errHandler != NULL)
	{
		delete errHandler;
	}

	if(parser != NULL)
	{
		delete parser;
	}


}


void xml_manager::writeXML(char *file)
{
	XMLFormatTarget *myFormTarget;
	myFormTarget = new LocalFileFormatTarget(file);

	XMLCh tempStr[100];
	XMLString::transcode("LS", tempStr, 99);
	DOMImplementation *impl          = DOMImplementationRegistry::getDOMImplementation(tempStr);
	DOMWriter         *writer = ((DOMImplementationLS*)impl)->createDOMWriter();

	//parser->setDocument(doc);
	//DOMNode *ndoc = parser->getDocument();

	//DOMWriter *writer = new DOMWriter();
	writer->setFeature( XMLUni::fgDOMWRTFormatPrettyPrint , true);

	writer->writeNode(myFormTarget, *doc );

	delete myFormTarget;
	delete writer;

}






//works on both users and usergroups
int xml_manager::delUser(char *username)
{

	//DOMNodeList *nodFullNames = 


	DOMNode *ndoc = doc->getLastChild();

	//DOMNode *ndoc2 = ndoc->getFirstChild();
	//std::cout<<"local name is "<<XMLString::transcode(ndoc->getNodeName())<<"\n";
	//DOMElement *root = doc->getDocumentElement();

	////std::cout<<"name: "<<XMLString::transcode(root->getLocalName())<<"\n";
	DOMNodeList *nlist = ndoc->getChildNodes();
	XMLSize_t numnodes = nlist->getLength();


	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;
	for(int i=0;i<numnodes;i++)
	{

		dnode = nlist->item(i);


		if(dnode->getNodeType()==1)
		{

			nnmap = dnode->getAttributes();


			XMLSize_t numat	= nnmap->getLength();

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
			// get attribute name
			name = XMLString::transcode(pAttributeNode->getValue());

			if(!strcmp(name,username))
			{

				i=numnodes;

			}	

		}
	}

	if(strcmp(name,username))
	{

		XMLString::release(&name);
		return USERNOTFOUND;

	}


	ndoc->removeChild(dnode);
	XMLString::release(&name);

	writeXML(filename);

	return SUCCESSFUL;

}




int xml_manager::updateUser(char *groupname,char *setting, char *value)
{

	//DOMNodeList *nodFullNames = 


	DOMNode *ndoc = doc->getLastChild();

	DOMNodeList *nlist = ndoc->getChildNodes();
	XMLSize_t numnodes = nlist->getLength();


	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;
	for(int i=0;i<numnodes;i++)
	{

		dnode = nlist->item(i);

		if(dnode->getNodeType()==1)
		{

			nnmap = dnode->getAttributes();


			XMLSize_t numat	= nnmap->getLength();

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_name"));
			// get attribute name
			name = XMLString::transcode(pAttributeNode->getValue());

			if(!strcmp(name,groupname))
			{
				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Lock_to_Group"));
				name = XMLString::transcode(pAttributeNode->getValue());
				if(!strcmp(name,"true"))
				{
					pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode(setting));
					pAttributeNode->setValue(XMLString::transcode(value));
				}
			}	

		}
	}



	XMLString::release(&name);

	writeXML(filename);

	return SUCCESSFUL;

}





bool xml_manager::checkIfGroupUsed(char *groupname)
{

	DOMNode *ndoc = doc->getLastChild();

	DOMNodeList *nlist = ndoc->getChildNodes();
	XMLSize_t numnodes = nlist->getLength();


	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;
	for(int i=0;i<numnodes;i++)
	{

		dnode = nlist->item(i);

		if(dnode->getNodeType()==1)
		{

			nnmap = dnode->getAttributes();

			XMLSize_t numat	= nnmap->getLength();

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_name"));
			// get attribute name
			name = XMLString::transcode(pAttributeNode->getValue());

			if(!strcmp(name,groupname))
			{
				XMLString::release(&name);
				return true;
			}	

		}
	}



	XMLString::release(&name);



	return false;

}







user* xml_manager::loadUser(char *username)
{

	//DOMNodeList *nodFullNames = 


	DOMNode *ndoc = doc->getLastChild();
	DOMNodeList *nlist = ndoc->getChildNodes();
	XMLSize_t numnodes = nlist->getLength();


	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;
	for(int i=0;i<numnodes;i++)
	{

		dnode = nlist->item(i);


		if(dnode->getNodeType()==1)
		{

			nnmap = dnode->getAttributes();


			XMLSize_t numat	= nnmap->getLength();

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
			// get attribute name
			name = XMLString::transcode(pAttributeNode->getValue());

			if(!strcmp(name,username))
			{

				i=numnodes;

			}	

		}
	}

	if(strcmp(name,username))
	{

		XMLString::release(&name);
		return NULL;

	}
	XMLString::release(&name);


	user *myUser = new user();

	myUser->setname(username);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Credits"));
	char *tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setcredits(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Download_Speed"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setmaxDownSpeed(atoi(tmp));
	XMLString::release(&tmp);

	//fix this one
	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Experation_date"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setexpDate(0);
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Logins"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setmaxLogins(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_D"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setratio_d(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bytes_uploaded"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setuploaded(_atoi64(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Check_IP_for_FXP"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setIPfxp(!strcmp(tmp,"true"));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Free_file_size"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setfreeFileSize(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_N"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setratio_n(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Upload_Speed"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setmaxUpSpeed(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bytes_downloaded"));
	tmp = XMLString::transcode(pAttributeNode->getValue());		
	myUser->setdownloaded(_atoi64(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Active"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setactive(!strcmp(tmp,"true"));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setBWlimit(atoi(tmp));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit_type"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setBWlimitType(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_this_session"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setbwthissession(_atoi64(tmp));
	XMLString::release(&tmp);




	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Reset_At"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setresetat(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_permissions"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setgroup(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Allow_Fxp"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setfxp(!strcmp(tmp,"true"));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Lock_to_Group"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setlocktogroup(!strcmp(tmp,"true"));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Password"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setpassword(tmp);
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_name"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUser->setgroupname(tmp);
	XMLString::release(&tmp);






	nlist = dnode->getChildNodes();
	numnodes = nlist->getLength();



	for(int i=0;i<numnodes;i++)
	{

		dnode = nlist->item(i);


		if(dnode->getNodeType()==1)
		{

			nnmap = dnode->getAttributes();

			XMLSize_t numat	= nnmap->getLength();
			//std::cout<<XMLString::transcode(dnode->getLocalName())<<"\n";

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Address"));
			tmp = XMLString::transcode(pAttributeNode->getValue());


			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Allow"));



			IPrules *ir = new IPrules(tmp,(!strcmp(XMLString::transcode(pAttributeNode->getValue()),"true")));
			IPrules *n = myUser->getiprules();
			XMLString::release(&tmp);
			if(n == NULL)
			{
				myUser->setiprules(ir);
			}
			else
			{

				while(n->getnext() != NULL)
				{
					n = n->getnext();
				};
				n->setnext(ir);
			}
		}
	}


	return myUser;	

}



int xml_manager::saveUser(user *myUser)
{

	DOMNode *ndoc = doc->getLastChild();

	DOMNodeList *nlist = ndoc->getChildNodes();
	XMLSize_t numnodes = nlist->getLength();


	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;

	for(int i=0;i<numnodes;i++)
	{

		dnode = nlist->item(i);


		if(dnode->getNodeType()==1)
		{

			nnmap = dnode->getAttributes();


			XMLSize_t numat	= nnmap->getLength();

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
			// get attribute name
			name = XMLString::transcode(pAttributeNode->getValue());

			if(!strcmp(name,myUser->getname()))
			{
				i=numnodes;

			}	

		}
	}

	if(strcmp(name,myUser->getname()))
	{
		XMLString::release(&name);
		return -1;

	}
	XMLString::release(&name);
	////std::cout<<"username here is "<<username<<"\n";
	char tmp[20];


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Credits"));
	_itoa_s(myUser->getcredits(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Download_Speed"));
	_itoa_s(myUser->getmaxDownSpeed(),tmp,20,10);      
	pAttributeNode->setValue(XMLString::transcode(tmp));


	//fix this one
	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Experation_date"));
	pAttributeNode->setValue(XMLString::transcode(""));
	myUser->getexpDate();


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Logins"));
	_itoa_s(myUser->getmaxLogins(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_D"));
	_itoa_s(myUser->getratio_d(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bytes_uploaded"));
	_itoa_s(myUser->getuploaded(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Check_IP_for_FXP"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUser->getIPfxp())));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Lock_to_Group"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUser->getlocktogroup())));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Free_file_size"));
	_itoa_s(myUser->getfreeFileSize(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));




	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_N"));
	_itoa_s(myUser->getratio_n(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));




	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Upload_Speed"));
	_itoa_s(myUser->getmaxUpSpeed(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bytes_downloaded"));
	_itoa_s(myUser->getdownloaded(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Active"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUser->getactive())));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit"));
	_itoa_s(myUser->getBWlimit(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Reset_At"));
	_itoa_s(myUser->getresetat(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_this_session"));
	_itoa_s(myUser->getbwthissession(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit_type"));
	_itoa_s(myUser->getBWlimitType(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_permissions"));
	_itoa_s(myUser->getgroup(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Allow_Fxp"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUser->getfxp())));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_name"));
	pAttributeNode->setValue(XMLString::transcode(myUser->getgroupname()));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Password"));
	pAttributeNode->setValue(XMLString::transcode(myUser->getpassword()));

	int cnt = 1;


	nlist = dnode->getChildNodes();
	DOMNode *dnode_old = dnode;
	numnodes = nlist->getLength();
	DOMNodeList *nlist_old = nlist;

	IPrules *n = myUser->getiprules();

	while(n->getnext() != NULL)
	{
		n = n->getnext();
		cnt++;
	}
	n = myUser->getiprules();



	if((cnt > numnodes/2))
	{
		int n=0;
		while(nlist->item(n)->getNodeType() != 1)
		{
			n++;
		};



		for(int i=0;i<(cnt - (numnodes/2));i++)
		{

			DOMNode *tnode = nlist->item(n)->cloneNode(true);	
			dnode->appendChild(tnode);
		}
	}
	////std::cout<<"the number if ip rules is "<<numiprules<<"\n";



	nlist = dnode->getChildNodes();
	numnodes = nlist->getLength();
	int i=0;
	for(i;i<numnodes;i++)
	{

		if(n == NULL)
		{
			break;
		}

		dnode = nlist->item(i);



		if(dnode->getNodeType()==1)
		{

			nnmap = dnode->getAttributes();


			XMLSize_t numat	= nnmap->getLength();

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Address"));
			pAttributeNode->setValue(XMLString::transcode(n->getipmask()));

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Allow"));




			n = n->getnext();

		}
	}



	for(i;i<numnodes-1;i++)
	{

		dnode = nlist_old->item(i);
		if(dnode->getNodeType()==1)
		{
			dnode_old->removeChild(dnode);
		}

	}


	writeXML(filename);

	return SUCCESSFUL;


}




int xml_manager::newUser(user *myUser)
{

	DOMNode *ndoc = doc->getLastChild();

	DOMNodeList *nlist = ndoc->getChildNodes();
	XMLSize_t numnodes = nlist->getLength();


	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;

	for(int i=0;i<numnodes;i++)
	{

		dnode = nlist->item(i);


		if(dnode->getNodeType()==1)
		{

			i=numnodes;
		}
	}



	char tmp[ITOABUFF];

	dnode_a =  dnode->cloneNode(true);

	nnmap = dnode_a->getAttributes();
	//std::cout<<"3\n";

	XMLSize_t numat	= nnmap->getLength();

	ndoc->appendChild(dnode_a);
	//DOMNode *dnode;
	nnmap = dnode->getAttributes();


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));

	pAttributeNode->setValue(XMLString::transcode(myUser->getname()));




	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Credits"));
	_itoa_s(myUser->getcredits(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Download_Speed"));
	_itoa_s(myUser->getmaxDownSpeed(),tmp,ITOABUFF,10);      
	pAttributeNode->setValue(XMLString::transcode(tmp));


	//fix this one
	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Experation_date"));
	pAttributeNode->setValue(XMLString::transcode(""));
	myUser->getexpDate();


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Logins"));
	_itoa_s(myUser->getmaxLogins(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_D"));
	_itoa_s(myUser->getratio_d(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bytes_uploaded"));
	_itoa_s(myUser->getuploaded(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Check_IP_for_FXP"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUser->getIPfxp())));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Free_file_size"));
	_itoa_s(myUser->getfreeFileSize(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));




	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_N"));
	_itoa_s(myUser->getratio_n(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));




	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Upload_Speed"));
	_itoa_s(myUser->getmaxUpSpeed(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bytes_downloaded"));
	_itoa_s(myUser->getdownloaded(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Active"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUser->getactive())));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit"));
	_itoa_s(myUser->getBWlimit(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Reset_At"));
	_itoa_s(myUser->getresetat(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_this_session"));
	_itoa_s(myUser->getbwthissession(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit_type"));
	_itoa_s(myUser->getBWlimitType(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_permissions"));
	_itoa_s(myUser->getgroup(),tmp,ITOABUFF,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Allow_Fxp"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUser->getfxp())));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Lock_to_Group"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUser->getlocktogroup())));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Password"));
	pAttributeNode->setValue(XMLString::transcode(myUser->getpassword()));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_name"));
	pAttributeNode->setValue(XMLString::transcode(myUser->getgroupname()));


	int cnt = 1;


	nlist = dnode->getChildNodes();
	DOMNode *dnode_old = dnode;
	numnodes = nlist->getLength();
	DOMNodeList *nlist_old = nlist;

	IPrules *n = myUser->getiprules();

	while(n->getnext() != NULL)
	{
		n = n->getnext();
		cnt++;
	}
	n = myUser->getiprules();

	int numiprules;

	if((cnt > numnodes/2))
	{
		int n=0;
		while(nlist->item(n)->getNodeType() != 1)
		{
			n++;
		};



		for(int i=0;i<(cnt - (numnodes/2));i++)
		{

			DOMNode *tnode = nlist->item(n)->cloneNode(true);	
			dnode->appendChild(tnode);
		}
	}
	////std::cout<<"the number if ip rules is "<<numiprules<<"\n";


	nlist = dnode->getChildNodes();
	numnodes = nlist->getLength();
	int i=0;
	for(i;i<numnodes;i++)
	{

		if(n == NULL)
		{
			break;
		}

		dnode = nlist->item(i);



		if(dnode->getNodeType()==1)
		{

			nnmap = dnode->getAttributes();


			XMLSize_t numat	= nnmap->getLength();
			//std::cout<<XMLString::transcode(dnode->getLocalName())<<"\n";

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Address"));
			//			//std::cout<<"the ip mask in the save is "<<n->getipmask()<<"\n";
			pAttributeNode->setValue(XMLString::transcode(n->getipmask()));

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Allow"));




			n = n->getnext();

		}
	}



	for(i;i<numnodes-1;i++)
	{

		dnode = nlist_old->item(i);
		if(dnode->getNodeType()==1)
		{
			dnode_old->removeChild(dnode);

		}

	}



	writeXML("user.xml");

	return 0;


}






ftp_options* xml_manager::loadOptions()
{

	DOMNode *ndoc = doc->getLastChild();

	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;

	nnmap = ndoc->getAttributes();


	XMLSize_t numat	= nnmap->getLength();

	ftp_options *myOptions = new ftp_options();
	char *tmp;


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
	tmp = XMLString::transcode(pAttributeNode->getValue());

	myOptions->setName(tmp);
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("UseSSL"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setusessl(!strcmp(tmp,"true"));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("maxusers"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setMaxusers(atoi(tmp));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("maxconnectionsperip"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setMaxconnectionsperip(atoi(tmp));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("idletimeout"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setIdletimeout(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("freefilesize"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setFreefilesize(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("dataportrangeL"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setDataportrangeL(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("dataportrangeH"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setDataportrangeH(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("port"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setPort(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("IP"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setIP(tmp);
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("packetsize"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setPacketsize(atoi(tmp));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setBWlimit(atoi(tmp));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit_type"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setBWlimitType(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_this_session"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setbwthissession(_atoi64(tmp));
	XMLString::release(&tmp);	

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Reset_At"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myOptions->setresetat(atoi(tmp));
	XMLString::release(&tmp);


	return myOptions;	

}





int xml_manager::saveOptions(ftp_options *myOptions)
{

	//DOMNodeList *nodFullNames = 


	DOMNode *ndoc = doc->getLastChild();

	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;

	nnmap = ndoc->getAttributes();


	XMLSize_t numat	= nnmap->getLength();



	char tmp[20];


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));


	pAttributeNode->setValue(XMLString::transcode(myOptions->getName()));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("maxusers"));
	_itoa_s(myOptions->getMaxusers(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("UseSSL"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myOptions->getusessl())));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("maxconnectionsperip"));
	_itoa_s(myOptions->getMaxconnectionsperip(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("idletimeout"));
	_itoa_s(myOptions->getIdletimeout(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("freefilesize"));
	_itoa_s(myOptions->getFreefilesize(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("dataportrangeL"));
	_itoa_s(myOptions->getDataportrangeL(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("dataportrangeH"));
	_itoa_s(myOptions->getDataportrangeH(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("port"));
	_itoa_s(myOptions->getPort(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("IP"));
	pAttributeNode->setValue(XMLString::transcode(myOptions->getIP()));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("packetsize"));
	_itoa_s(myOptions->getPacketsize(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit"));
	_itoa_s(myOptions->getBWlimit(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Reset_At"));
	_itoa_s(myOptions->getresetat(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_this_session"));
	_itoa_s(myOptions->getbwthissession(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit_type"));
	_itoa_s(myOptions->getBWlimitType(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	writeXML(filename);
	return SUCCESSFUL;

}






folder* xml_manager::loadDir()
{

	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	DOMAttr *pAttributeNode2;


	DOMNode *ndoc = doc->getLastChild();

	DOMNodeList *nlist = ndoc->getChildNodes();

	nnmap = ndoc->getAttributes();

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Location"));
	pAttributeNode2 = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));

	folder *dtree = new folder(XMLString::transcode(pAttributeNode->getValue()),XMLString::transcode(pAttributeNode2->getValue()));


	////std::cout<<"name: "<<XMLString::transcode(root->getLocalName())<<"\n";
	XMLSize_t numnodes = nlist->getLength();

	for(int i=0;i<numnodes;i++)
	{
		////std::cout<<"1\n";
		dnode = nlist->item(i);
		////std::cout<<"2\n";

		if(dnode->getNodeType()==1)
		{


			if(!strcmp(XMLString::transcode(dnode->getLocalName()),"Sub"))
			{
				nnmap = dnode->getAttributes();
				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Location"));
				pAttributeNode2 = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
				DOMNodeList *nlist2 = dnode->getChildNodes();
				folder *fldr = procFolder(nlist2);

				fldr->setName(XMLString::transcode(pAttributeNode2->getValue()));
				//;



				fldr->setLocation(XMLString::transcode(pAttributeNode->getValue()));

				file *tmpfile = fldr->getFfirst();
				while(tmpfile != NULL)
				{
					char buf[255];
					sprintf_s(buf,255,"%s%s",fldr->getLocation(),tmpfile->getPath());
					////std::cout<<"the dir is: "<<buf<<"\n";
					bool DELFLAG = false;

					if(tmpfile->getDir())
					{
						if(!checkRealdir(buf, tmpfile->getName()))
						{
							DELFLAG = true;	
						}
					}
					else
					{
						if(!checkfile(buf, tmpfile->getName()))
						{
							DELFLAG = true;	
						}

					}

					if(DELFLAG)
					{

						//setting the next value of the node behind this one to the node infront of this one
						//aka removing this node from the list
						if(tmpfile->getLast() != NULL)
						{
							tmpfile->getLast()->setNext(tmpfile->getNext());
						}
						else
						{
							fldr->setFfirst(tmpfile->getNext());
						}
						//setting the value of last on the node infront of this one to the node behind this one
						//aka removing the node from the list
						if(tmpfile->getNext() != NULL)
						{
							tmpfile->getNext()->setLast(tmpfile->getLast());
						}
						//storing the pointer to this node so we can delete it
						file *tmpfile_d = tmpfile;
						//moving to the next node in the list
						tmpfile = tmpfile->getNext();
						//deleteing the node to the file that does not exist

						delete tmpfile_d;

					}
					else
					{
						tmpfile = tmpfile->getNext();
					}
				};


				if(dtree->getDown() == NULL)
				{

					dtree->setDown(fldr);
					fldr->setUp(dtree);

					fldr->setLast(NULL);

				}
				else
				{
					folder *tmp;
					tmp = dtree->getDown();
					while (tmp->getNext() != NULL)
					{
						tmp = tmp->getNext();
					};
					tmp->setNext(fldr);
					fldr->setLast(tmp);
					fldr->setUp(dtree);
				}

			}




			if(!strcmp(XMLString::transcode(dnode->getLocalName()),"Group"))
			{
				nnmap = dnode->getAttributes();
				group *tempgroup = new group();
				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
				tempgroup->setName(atoi(XMLString::transcode(pAttributeNode->getValue())));

				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Permissions"));
				tempgroup->setPremissions(atoi(XMLString::transcode(pAttributeNode->getValue())));

				if(dtree->getGfirst() == NULL)
				{

					dtree->setGfirst(tempgroup);
				}
				else
				{

					group *tmp;
					tmp = dtree->getGfirst();

					while (tmp->getNext() != NULL)
					{
						if(tmp != NULL)
						{
							tmp = tmp->getNext();
						}

					};
					tmp->setNext(tempgroup);
					tempgroup->setLast(tmp);
				}

			}

			if(!strcmp(XMLString::transcode(dnode->getLocalName()),"File"))
			{
				nnmap = dnode->getAttributes();

				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
				char *tmp = XMLString::transcode(pAttributeNode->getValue());

				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Path"));
				XMLString::transcode(pAttributeNode->getValue());
				char *tmp2 = XMLString::transcode(pAttributeNode->getValue());


				char buf[255];
				sprintf_s(buf,255,"%s%s",dtree->getLocation(),tmp2);

				sprintf_s(buf,255,"%s%s",dtree->getLocation(),tmp2);
				////std::cout<<"the dir is: "<<buf<<"\n";
				if(checkfile(buf, tmp))
				{

					file *tempfile = new file();

					pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Dir"));
					tempfile->setDir((!strcmp(XMLString::transcode(pAttributeNode->getValue()),"true")));

					pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Owner"));
					tempfile->setOwner(XMLString::transcode(pAttributeNode->getValue()));
					tempfile->setName(tmp);
					tempfile->setPath(tmp2);

					pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group"));
					tempfile->setGroup(atoi(XMLString::transcode(pAttributeNode->getValue())));


					if(dtree->getFfirst() == NULL)
					{

						dtree->setFfirst(tempfile);
					}
					else
					{

						file *tmp;
						tmp = dtree->getFfirst();

						while (tmp->getNext() != NULL)
						{
							if(tmp != NULL)
							{
								tmp = tmp->getNext();
							}

						};
						tmp->setNext(tempfile);
						tempfile->setLast(tmp);
					}



				}
				XMLString::release(&tmp);
				XMLString::release(&tmp2);

			}


		}
	}


	//XMLString::release(&name);
	return dtree;
}





int xml_manager::saveDir(folder *dtree)
{


	DOMImplementation *impl =  DOMImplementationRegistry::getDOMImplementation(X("Core"));

	if (impl != NULL)
	{


		XERCES_CPP_NAMESPACE::DOMDocument *mydoc = impl->createDocument(
			0,                    // root element namespace URI.
			X("Folder"),         // root element name
			0);                   // document type object (DTD).

		DOMElement* rootElem = mydoc->getDocumentElement();
		rootElem->setAttribute(X("Location"), X(dtree->getLocation()));
		rootElem->setAttribute(X("Name"), X(dtree->getName()));

		group *tgroup = dtree->getGfirst();

		while(tgroup != NULL)
		{

			char tmp[20];
			DOMElement*  gElem = mydoc->createElement(X("Group"));
			rootElem->appendChild(gElem);
			_itoa_s(tgroup->getName(),tmp,20,10);
			gElem->setAttribute(X("Name"), X(tmp));
			_itoa_s(tgroup->getPremissions(),tmp,20,10);
			gElem->setAttribute(X("Permissions"), X(tmp));
			tgroup = tgroup->getNext();
		};



		file *tfile = dtree->getFfirst();

		while(tfile != NULL)
		{
			char tmp[20];
			DOMElement*  fElem = mydoc->createElement(X("File"));
			rootElem->appendChild(fElem);
			_itoa_s(tfile->getGroup(),tmp,20,10);
			fElem->setAttribute(X("Group"), X(tmp));

			fElem->setAttribute(X("Dir"), X(getTF(tfile->getDir())));
			fElem->setAttribute(X("Name"), X(tfile->getName()));
			fElem->setAttribute(X("Owner"), X(tfile->getOwner()));
			fElem->setAttribute(X("Path"), X(tfile->getPath()));
			tfile = tfile->getNext();
		};



		folder *tfldr = dtree->getDown();

		while(tfldr != NULL)
		{
			char tmp[20];
			DOMElement*  sElem = mydoc->createElement(X("Sub"));
			rootElem->appendChild(sElem);
			procSaveFolder(sElem,tfldr,mydoc);

			tfldr = tfldr->getNext();
		};


		XMLFormatTarget *myFormTarget;
		myFormTarget = new LocalFileFormatTarget("dir.xml");

		XMLCh tempStr[100];
		XMLString::transcode("LS", tempStr, 99);
		DOMImplementation *impl          = DOMImplementationRegistry::getDOMImplementation(tempStr);
		DOMWriter         *writer = ((DOMImplementationLS*)impl)->createDOMWriter();

		//parser->setDocument(doc);
		//DOMNode *ndoc = parser->getDocument();

		//DOMWriter *writer = new DOMWriter();
		writer->setFeature( XMLUni::fgDOMWRTFormatPrettyPrint , true);

		writer->writeNode(myFormTarget, *mydoc );

		delete myFormTarget;
		delete writer;






		mydoc->release();

	}  // (inpl != NULL)



	return 0;

}

void xml_manager::procSaveFolder(DOMElement*  sElem,folder *fldr, XERCES_CPP_NAMESPACE::DOMDocument *mydoc)
{


	sElem->setAttribute(X("Location"), X(fldr->getLocation()));
	sElem->setAttribute(X("Name"), X(fldr->getName()));

	group *tgroup = fldr->getGfirst();

	while(tgroup != NULL)
	{
		char tmp[20];
		DOMElement*  gElem = mydoc->createElement(X("Group"));
		sElem->appendChild(gElem);
		_itoa_s(tgroup->getName(),tmp,20,10);
		gElem->setAttribute(X("Name"), X(tmp));
		_itoa_s(tgroup->getPremissions(),tmp,20,10);
		gElem->setAttribute(X("Permissions"), X(tmp));
		tgroup = tgroup->getNext();
	};



	file *tfile = fldr->getFfirst();

	while(tfile != NULL)
	{
		char tmp[20];
		DOMElement*  fElem = mydoc->createElement(X("File"));
		sElem->appendChild(fElem);
		_itoa_s(tfile->getGroup(),tmp,20,10);
		fElem->setAttribute(X("Group"), X(tmp));
		fElem->setAttribute(X("Dir"), X(getTF(tfile->getDir())));			  
		fElem->setAttribute(X("Name"), X(tfile->getName()));
		fElem->setAttribute(X("Owner"), X(tfile->getOwner()));
		fElem->setAttribute(X("Path"), X(tfile->getPath()));
		tfile = tfile->getNext();
	};



	folder *tfldr = fldr->getDown();

	while(tfldr != NULL)
	{
		char tmp[20];
		DOMElement*  sElem2 = mydoc->createElement(X("Sub"));
		sElem->appendChild(sElem2);
		procSaveFolder(sElem2,tfldr,mydoc);

		tfldr = tfldr->getNext();
	};


}




folder* xml_manager::procFolder(DOMNodeList *nlist)
{

	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	DOMAttr *pAttributeNode2;


	folder *fldr = new folder();	

	int numnodes = nlist->getLength();

	for(int i=0;i<numnodes;i++)
	{
		////std::cout<<"1\n";
		dnode = nlist->item(i);
		////std::cout<<"2\n";

		if(dnode->getNodeType()==1)
		{



			if(!strcmp(XMLString::transcode(dnode->getLocalName()),"Sub"))
			{

				nnmap = dnode->getAttributes();
				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Location"));

				pAttributeNode2 = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));

				folder *fldr2 = procFolder(dnode->getChildNodes());



				fldr2->setName(XMLString::transcode(pAttributeNode2->getValue()));
				fldr2->setLocation(XMLString::transcode(pAttributeNode->getValue()));




				file *tmpfile = fldr2->getFfirst();
				while(tmpfile != NULL)
				{
					char buf[255];
					sprintf_s(buf,255,"%s%s",fldr2->getLocation(),tmpfile->getPath());
					////std::cout<<"the dir is: "<<buf<<" name is "<<tmpfile->getName()<<"\n";

					bool DELFLAG = false;

					if(tmpfile->getDir())
					{
						if(!checkRealdir(buf, tmpfile->getName()))
						{
							DELFLAG = true;	
						}
					}
					else
					{
						if(!checkfile(buf, tmpfile->getName()))
						{
							DELFLAG = true;	
						}

					}

					if(DELFLAG)
					{

						//setting the next value of the node behind this one to the node infront of this one
						//aka removing this node from the list
						if(tmpfile->getLast() != NULL)
						{
							tmpfile->getLast()->setNext(tmpfile->getNext());
						}
						else
						{
							fldr2->setFfirst(tmpfile->getNext());
						}
						//setting the value of last on the node infront of this one to the node behind this one
						//aka removing the node from the list
						if(tmpfile->getNext() != NULL)
						{
							tmpfile->getNext()->setLast(tmpfile->getLast());
						}
						//storing the pointer to this node so we can delete it
						file *tmpfile_d = tmpfile;
						//moving to the next node in the list
						tmpfile = tmpfile->getNext();
						//deleteing the node to the file that does not exist

						delete tmpfile_d;

					}
					else
					{
						tmpfile = tmpfile->getNext();
					}

				};


				if(fldr->getDown() == NULL)
				{

					fldr->setDown(fldr2);
					fldr2->setUp(fldr);

					fldr2->setLast(NULL);

				}
				else
				{
					folder *tmp;
					tmp = fldr->getDown();
					while (tmp->getNext() != NULL)
					{
						tmp = tmp->getNext();
					};
					tmp->setNext(fldr2);
					fldr2->setLast(tmp);
					fldr2->setUp(fldr);
				}

			}




			if(!strcmp(XMLString::transcode(dnode->getLocalName()),"Group"))
			{
				nnmap = dnode->getAttributes();
				group *tempgroup = new group();
				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
				tempgroup->setName(atoi(XMLString::transcode(pAttributeNode->getValue())));

				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Permissions"));
				tempgroup->setPremissions(atoi(XMLString::transcode(pAttributeNode->getValue())));

				if(fldr->getGfirst() == NULL)
				{
					fldr->setGfirst(tempgroup);
				}
				else
				{
					group *tmp;
					tmp = fldr->getGfirst();
					while (tmp->getNext() != NULL)
					{
						tmp = tmp->getNext();
					};
					tmp->setNext(tempgroup);
					tempgroup->setLast(tmp);
				}

			}

			if(!strcmp(XMLString::transcode(dnode->getLocalName()),"File"))
			{
				nnmap = dnode->getAttributes();
				file *tempfile = new file();
				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
				tempfile->setName(XMLString::transcode(pAttributeNode->getValue()));

				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Path"));

				tempfile->setPath(XMLString::transcode(pAttributeNode->getValue()));


				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Owner"));
				tempfile->setOwner(XMLString::transcode(pAttributeNode->getValue()));

				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Dir"));
				tempfile->setDir( (!strcmp(XMLString::transcode(pAttributeNode->getValue()),"true")) );

				pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group"));
				tempfile->setGroup(atoi(XMLString::transcode(pAttributeNode->getValue())));


				if(fldr->getFfirst() == NULL)
				{

					fldr->setFfirst(tempfile);
				}
				else
				{
					//	//std::cout<<"ffirst = "<<fldr->getFfirst()<<"\n";
					file *tmp;
					tmp = fldr->getFfirst();
					////std::cout<<"tmp = "<<tmp<<"\n";
					while (tmp->getNext() != NULL)
					{
						if(tmp != NULL)
						{
							tmp = tmp->getNext();
						}
						////std::cout<<"tmp = "<<tmp<<"\n";
					};
					tmp->setNext(tempfile);
					tempfile->setLast(tmp);
				}

			}






		}
	}

	return fldr;


}





usergroup* xml_manager::loadGroup(char *groupname)
{

	//DOMNodeList *nodFullNames = 


	DOMNode *ndoc = doc->getLastChild();

	DOMNodeList *nlist = ndoc->getChildNodes();
	XMLSize_t numnodes = nlist->getLength();


	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;
	for(int i=0;i<numnodes;i++)
	{

		dnode = nlist->item(i);


		if(dnode->getNodeType()==1)
		{

			nnmap = dnode->getAttributes();


			XMLSize_t numat	= nnmap->getLength();

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
			// get attribute name
			name = XMLString::transcode(pAttributeNode->getValue());

			if(!strcmp(name,groupname))
			{
				i=numnodes;

			}	

		}
	}

	if(strcmp(name,groupname))
	{
		XMLString::release(&name);
		return NULL;

	}
	XMLString::release(&name);
	//	//std::cout<<"username here is "<<username<<"\n";

	usergroup *myUsergroup = new usergroup();

	myUsergroup->setname(groupname);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Download_Speed"));
	char *tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setmaxDownSpeed(atoi(tmp));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Logins"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setmaxLogins(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_D"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setratio_d(atoi(tmp));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Check_IP_for_FXP"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setIPfxp(!strcmp(tmp,"true"));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Free_file_size"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setfreeFileSize(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_N"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setratio_n(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Upload_Speed"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setmaxUpSpeed(atoi(tmp));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setBWlimit(atoi(tmp));
	XMLString::release(&tmp);


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit_type"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setBWlimitType(atoi(tmp));
	XMLString::release(&tmp);



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_Permissions"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setgroup(atoi(tmp));
	XMLString::release(&tmp);

	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Allow_Fxp"));
	tmp = XMLString::transcode(pAttributeNode->getValue());
	myUsergroup->setfxp(!strcmp(tmp,"true"));
	XMLString::release(&tmp);


	return myUsergroup;	

}







int xml_manager::saveUsergroup(usergroup *myUsergroup)
{

	DOMNode *ndoc = doc->getLastChild();

	DOMNodeList *nlist = ndoc->getChildNodes();
	XMLSize_t numnodes = nlist->getLength();


	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;

	for(int i=0;i<numnodes;i++)
	{

		dnode = nlist->item(i);


		if(dnode->getNodeType()==1)
		{

			nnmap = dnode->getAttributes();


			XMLSize_t numat	= nnmap->getLength();

			pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));
			// get attribute name
			name = XMLString::transcode(pAttributeNode->getValue());

			if(!strcmp(name,myUsergroup->getname()))
			{

				i=numnodes;

			}	

		}
	}

	if(strcmp(name,myUsergroup->getname()))
	{

		return -1;

	}
	XMLString::release(&name);
	////std::cout<<"username here is "<<username<<"\n";
	char tmp[20];


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Download_Speed"));
	_itoa_s(myUsergroup->getmaxDownSpeed(),tmp,20,10);      
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Logins"));
	_itoa_s(myUsergroup->getmaxLogins(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_D"));
	_itoa_s(myUsergroup->getratio_d(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Check_IP_for_FXP"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUsergroup->getIPfxp())));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Free_file_size"));
	_itoa_s(myUsergroup->getfreeFileSize(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));




	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_N"));
	_itoa_s(myUsergroup->getratio_n(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));




	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Upload_Speed"));
	_itoa_s(myUsergroup->getmaxUpSpeed(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit"));
	_itoa_s(myUsergroup->getBWlimit(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit_type"));
	_itoa_s(myUsergroup->getBWlimitType(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_Permissions"));
	_itoa_s(myUsergroup->getgroup(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Allow_Fxp"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUsergroup->getfxp())));




	writeXML(filename);

	return SUCCESSFUL;


}









int xml_manager::newUsergroup(usergroup *myUsergroup)
{

	DOMNode *ndoc = doc->getLastChild();


	DOMNodeList *nlist = ndoc->getChildNodes();
	XMLSize_t numnodes = nlist->getLength();


	DOMNode *dnode;
	DOMNode *dnode_a;
	DOMNamedNodeMap *nnmap;
	DOMAttr *pAttributeNode;
	char *name;

	for(int i=0;i<numnodes;i++)
	{

		dnode = nlist->item(i);


		if(dnode->getNodeType()==1)
		{

			//std::cout<<"we have a match\n";
			i=numnodes;

		}
	}


	char tmp[ITOABUFF];

	dnode_a =  dnode->cloneNode(true);

	nnmap = dnode_a->getAttributes();
	//std::cout<<"3\n";

	XMLSize_t numat	= nnmap->getLength();

	ndoc->appendChild(dnode_a);
	//DOMNode *dnode;
	nnmap = dnode->getAttributes();


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Name"));

	pAttributeNode->setValue(XMLString::transcode(myUsergroup->getname()));


	//	XMLString::release(&name);
	////std::cout<<"username here is "<<username<<"\n";



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Download_Speed"));
	_itoa_s(myUsergroup->getmaxDownSpeed(),tmp,20,10);      
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Logins"));
	_itoa_s(myUsergroup->getmaxLogins(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_D"));
	_itoa_s(myUsergroup->getratio_d(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Check_IP_for_FXP"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUsergroup->getIPfxp())));


	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Free_file_size"));
	_itoa_s(myUsergroup->getfreeFileSize(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));




	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Ratio_N"));
	_itoa_s(myUsergroup->getratio_n(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));




	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Max_Upload_Speed"));
	_itoa_s(myUsergroup->getmaxUpSpeed(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit"));
	_itoa_s(myUsergroup->getBWlimit(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Bandwidth_limit_type"));
	_itoa_s(myUsergroup->getBWlimitType(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Group_Permissions"));
	_itoa_s(myUsergroup->getgroup(),tmp,20,10);
	pAttributeNode->setValue(XMLString::transcode(tmp));



	pAttributeNode = (DOMAttr*) nnmap->getNamedItem(XMLString::transcode("Allow_Fxp"));
	pAttributeNode->setValue(XMLString::transcode(getTF(myUsergroup->getfxp())));



	writeXML("group.xml");

	return 0;


}
